Type: Package
Package: ceRNAnetsim
Title: Regulation Simulator of Interaction between miRNA and Competing
        RNAs (ceRNA)
Version: 1.23.0
Authors@R: 
    c(person(given = "Selcen",
             family = "Ari Yuka",
             role = c("aut", "cre"),
             email = "selcenarii@gmail.com",
             comment = c(ORCID = "0000-0002-0028-2453")),
      person(given = "Alper",
             family = "Yilmaz",
             role = "aut",
             email = "alperyilmaz@gmail.com",
             comment = c(ORCID = "0000-0002-8827-4887")))
Description: This package simulates regulations of ceRNA
    (Competing Endogenous) expression levels after a expression level
    change in one or more miRNA/mRNAs. The methodolgy adopted by the
    package has potential to incorparate any ceRNA (circRNA, lincRNA,
    etc.) into miRNA:target interaction network.  The package basically
    distributes miRNA expression over available ceRNAs where each ceRNA
    attracks miRNAs proportional to its amount. But, the package can
    utilize multiple parameters that modify miRNA effect on its target
    (seed type, binding energy, binding location, etc.).  The functions
    handle the given dataset as graph object and the processes progress
    via edge and node variables.
License: GPL (>= 3.0)
URL: https://github.com/selcenari/ceRNAnetsim
BugReports: https://github.com/selcenari/ceRNAnetsim/issues
Depends: R (>= 4.0.0), dplyr, tidygraph
Imports: furrr, rlang, tibble, ggplot2, ggraph, igraph, purrr, tidyr,
        future, stats
Suggests: knitr, png, rmarkdown, testthat, covr
VignetteBuilder: knitr
biocViews: NetworkInference, SystemsBiology, Network, GraphAndNetwork,
        Transcriptomics
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/ceRNAnetsim
git_branch: devel
git_last_commit: 4b53125
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:32:34 UTC; biocbuild
Author: Selcen Ari Yuka [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0028-2453>),
  Alper Yilmaz [aut] (ORCID: <https://orcid.org/0000-0002-8827-4887>)
Maintainer: Selcen Ari Yuka <selcenarii@gmail.com>

# Generated by roxygen2: do not edit by hand

export(calc_perturbation)
export(find_affected_nodes)
export(find_iteration)
export(find_node_perturbation)
export(find_targeting_nodes)
export(priming_graph)
export(simulate)
export(simulate_vis)
export(update_how)
export(update_nodes)
export(update_variables)
export(vis_graph)
import(dplyr)
import(tidygraph)
importFrom(furrr,future_map)
importFrom(furrr,future_map_dfr)
importFrom(future,plan)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggraph,create_layout)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,ggraph)
importFrom(ggraph,theme_graph)
importFrom(igraph,'E<-')
importFrom(igraph,'V<-')
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,edge)
importFrom(igraph,edges)
importFrom(igraph,gsize)
importFrom(purrr,map_dbl)
importFrom(purrr,pmap)
importFrom(purrr,reduce)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,set_names)
importFrom(stats,na.omit)
importFrom(tibble,tibble)
importFrom(tidygraph,as_tibble)
importFrom(tidyr,unnest)
importFrom(utils,head)

## ceRNAnetsim

<!-- badges: start -->
[![License: GPL v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](http://www.gnu.org/licenses/gpl-3.0) [![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/selcenari/ceRNAnetsim/services?urlpath=rstudio) 
[![Travis build status](https://travis-ci.org/selcenari/ceRNAnetsim.svg?branch=master)](https://travis-ci.org/selcenari/ceRNAnetsim)
[![Codecov test coverage](https://codecov.io/gh/selcenari/ceRNAnetsim/branch/master/graph/badge.svg)](https://codecov.io/gh/selcenari/ceRNAnetsim?branch=master)
<!-- badges: end -->


<p align="center">
  <img width="500" height="300" src="https://media.giphy.com/media/l0ErNdz1w5vt3YdZm/giphy.gif">
</p>

`ceRNAnetsim` helps to understanding complex miRNA:target interactions with the network based approach. It utilizes the amounts of miRNAs and their targets and interaction parameters optionally.


## Installation

Installation of `ceRNAnetsim`:

```R
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("ceRNAnetsim")

library(ceRNAnetsim)

```

## Example

```
data("minsamp")

minsamp

priming_graph(minsamp, 
                competing_count = Competing_expression, 
                miRNA_count = miRNA_expression)

```


## Local Files

The `ceRNAnetsim` provides various datasets for experimenting with package functions. These are:

- *minsamp* : the minimum sample consists of 2 miRNAs and 6 genes which are targeted by them. *new_counts* is a helper dataset for *minsamp*.
- *midsamp* : middle-sized sample is a network of 4 miRNAs and 20 competing target genes.
- *huge_example* : this dataset is comprised of a large network which incorporates three datasets: 
  - gene expression levels (RNA-Seq) retrieved from [TCGA-BRCA](https://portal.gdc.cancer.gov/projects?filters=%7B%22op%22%3A%22and%22%2C%22content%22%3A%5B%7B%22op%22%3A%22in%22%2C%22content%22%3A%7B%22field%22%3A%22projects.project_id%22%2C%22value%22%3A%5B%22TCGA-BRCA%22%5D%7D%7D%5D%7D) breast cancer 
  - miRNA expression of a breast cancer patient (from TCGA)
  - and the miRNA:target dataset gathered from two different high-throughput experimental studies. ([CLASH](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3650559/) & [CLEAR-CLiP](https://www.nature.com/articles/ncomms9864) methods)
- The remaining datasets are used as reproducible example of methods shown as [Mirtarbase example](https://selcenari.github.io/ceRNAnetsim/articles/mirtarbase_example.html)

## Vignettes

The repository of ceRNAnetsim contents the vignettes that follow:

- [What does ceRNAnetsim do?](https://selcenari.github.io/ceRNAnetsim/articles/basic_usage.html)
- [How to calculate proper number of iterations for simulation of the network](https://selcenari.github.io/ceRNAnetsim/articles/convenient_iteration.html)
- [How does the system do in a real-world network?](https://selcenari.github.io/ceRNAnetsim/articles/mirtarbase_example.html)
- [Helper functions for users.](https://selcenari.github.io/ceRNAnetsim/articles/auxiliary_commands.html)

Also see [package webpage](https://selcenari.github.io/ceRNAnetsim/)


## Issues

Contact us for any issue or bug via:

- [Issues](https://github.com/selcenari/ceRNAnetsim/issues)
- [Alper Yilmaz](mailto:alperyilmaz@gmail.com)
- [Selcen Ari](mailto:selcenarii@gmail.com)

#' Calculates average expression changes of all (or specified)
#' nodes except trigger and finds the perturbed node count for
#' all (or specified) nodes in system.
#'
#'
#' @importFrom furrr future_map future_map_dfr
#' @importFrom future plan
#' @importFrom rlang set_names
#' @importFrom igraph gsize degree V 'V<-'
#' @importFrom purrr map_dbl map
#' @importFrom tidyr unnest
#'
#' @return It gives a tibble form dataset that includes node names,
#' perturbation efficiency and perturbed count of nodes.
#'
#' @details find_node_perturbation calculates mean expression changes
#' of elements after the change in the network in terms of percentage.
#' It also calculates the number of nodes that have expression changes
#' after the change occur in the network. The outputs of the function
#' are the perturbation efficiency and perturbed count of nodes for
#' each nodes.
#'
#' @param input_graph The graph object that was processed with priming_graph function.
#' @param how The change of count (expression) of the given node in terms of fold change.
#' @param cycle The iteration of simulation.
#' @param limit The minimum fold change which can be taken into account for
#' perturbation calculation on all nodes in terms of percentage.
#' @param fast specifies percentage of affected target in target expression.
#' For example, if fast = 1, the nodes that are affected from miRNA repression
#' activity more than one percent of their expression is determined as subgraph.
#'
#' @examples
#'
#' data('minsamp')
#' data('midsamp')
#'
#'  minsamp%>%
#'   priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)%>%
#'   find_node_perturbation()%>%
#'   select(name, perturbation_efficiency, perturbed_count)
#'
#'
#'  minsamp%>%
#'   priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression,
#'     aff_factor = c(energy,seed_type), deg_factor = region)%>%
#'   find_node_perturbation(how = 3, cycle = 4)%>%
#'   select(name, perturbation_efficiency, perturbed_count)
#'
#'  midsamp%>%
#'   priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
#'   find_node_perturbation(how = 2, cycle= 3, limit=1, fast = 5)%>%
#'   select(name, perturbation_efficiency, perturbed_count)
#'
#'
#' @export

find_node_perturbation <- function(input_graph, how = 2, cycle = 1, limit = 0, fast = 0) {
  
  if (fast == 0) {
    
    result <- input_graph %>% 
      activate(nodes) %>% 
      tidygraph::mutate(eff_count = future_map(V(input_graph)$name, ~calc_perturbation(input_graph, .x, how, cycle, limit))) %>%
      tibble::as_tibble() %>% 
      unnest(eff_count)
  }
  
  if (fast != 0) {
    
    input_graph %>% 
      tidygraph::activate(edges) -> main_graph
    
    main_graph %>% 
      tidygraph::mutate(fast = ifelse(100*effect_current/comp_count_current > fast, TRUE, FALSE)) %>%
      tidygraph::filter(fast) %>%
      tidygraph::activate(nodes) %>%
      tidygraph::mutate(degree = tidygraph::centrality_degree(mode = "all")) %>%
      tidygraph::filter(degree > 0) -> fast_graph
    
    fast_graph %>% 
      tidygraph::mutate(eff_count = purrr::map(name, ~calc_perturbation(fast_graph, .x,  how, cycle, limit))) %>% 
      activate(nodes) %>% 
      as_tibble() %>% 
      unnest() %>% 
      dplyr::select(name, perturbation_efficiency, perturbed_count)-> res_calc_per
    
    result<- main_graph %>% 
      activate(nodes) %>% 
      as_tibble() %>% 
      left_join(res_calc_per, by = "name")
    
  }

  
  return(result)
  
}

context("Simulation and perturbation of network")

test_that("Could the functions work? ",{

  data("midsamp")
  data("minsamp")

calc_test<- midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    calc_perturbation(sample_n(midsamp, 1)$Genes,3, cycle = 30, limit = 0.1)

perturb_test<- midsamp%>%
  priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)

midsamp %>%
  priming_graph(competing_count = Gene_expression,
                miRNA_count = miRNA_expression) %>%
  update_how(node_name = sample_n(midsamp, 1)$Genes, how= 3) %>%
  simulate(10) %>%
  find_iteration() -> midsamp_iteration_test

minsamp%>%
  priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)%>%
  find_node_perturbation(how=5, cycle=3, fast = 3)%>%
  select(perturbation_efficiency)%>%
  pull()%>%
  round(digits = 2)-> node_efficiencies

  expect_is(midsamp_iteration_test, 'numeric')
  expect_gt(midsamp_iteration_test,  0)
  expect_equal(attr(calc_test, "names"), c("perturbation_efficiency", "perturbed_count" ))
  expect_true(is.data.frame(find_node_perturbation(perturb_test)))
  expect_equal(node_efficiencies, c(NA, NA, NA, 3.18, 2.27, 3.18, NA, 32.00))
  expect_error(midsamp %>%
                 priming_graph(competing_count = Gene_expression,
                               miRNA_count = miRNA_expression) %>%
                 update_how(node_name = sample_n(midsamp, 1)$Genes, how= 3) %>%
                 simulate(cycle = 0),
               "number of cycles should be more than 1")

 })



test_that("Outputs of the functions", {

data("midsamp")

  classofout<- midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    update_how(sample_n(midsamp, 1)$Genes, 3)%>%
    simulate(15)%>%
    class()


  classofout2<-midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    update_how(sample_n(midsamp, 1)$Genes, 3)%>%
    simulate(4)%>%
    class()

  expect_equal(classofout, classofout2)


})

#' Replaces new values with previous values of
#' competing or miRNA counts.
#'
#' This function replaces new values with previous
#' values of competing or miRNA counts.
#'
#' @importFrom igraph E 'E<-'
#' @return the graph object.
#'
#' @details update_variables function provides updating edge
#' variables to current values. If the microRNA or competing
#' expression (or both) change (decreasing or increasing),
#' this function switches the values that are found in a new
#' dataset provided by user. But the current value dataset
#' must be equal with initial dataset in terms of node name.
#'
#' @param input_graph The processed graph object.
#' @param current_counts The additional df that provided by user.
#'
#' @examples
#'
#' data('minsamp')
#' data('new_counts')
#'
#'  minsamp%>%
#'  priming_graph(Competing_expression, miRNA_expression,
#'      aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'    update_variables(new_counts)
#'    #new_counts includes the current counts of nodes.
#'
#' @export

update_variables <- function(input_graph, current_counts) {

  if (isTRUE(all.equal(current_counts$Competing, E(input_graph)$Competing_name))) {

    E(input_graph)$comp_count_current[match(current_counts$Competing,
                                            E(input_graph)$Competing_name)] <- current_counts$Competing_count
  } else {
    stop("Current values include one or more different node name!",
         call. = FALSE)

  }

  if (isTRUE(all.equal(current_counts$miRNA, E(input_graph)$miRNA_name))) {

    E(input_graph)$mirna_count_current[match(current_counts$miRNA,
                                             E(input_graph)$miRNA_name)] <- current_counts$miRNA_count
  } else {
    stop("Current values include one or more different node name!",
         call. = FALSE)

  }

  return(input_graph %>% update_nodes())

}

context("Convertion to the graph.")


testdata <- data.frame(competing = c("TP53", "Gene2", "ABCC1", "Gene4", "Gene4", "Gene5", "PTEN"),
                       miRNA = c("mir1", "mir1","hsa-mir3", "mir1", "MiR2", "MiR2", "miR4"),
                       Competing_expression = c(10000, 10000, 5000, 10000, 10000, 5000, 10000),
                       miRNA_expression = c(1000, 1000, 1000, 1000, 2000, 2000, 2000),
                       stringsAsFactors = FALSE)

testdata2 <- data.frame(competing = c("TP53", "Gene-2", "ABCC1", "Gene 4", "Gene 4", "Gene5", "PTEN"),
                        miRNA = c("mir1", "mir1","hsa-mir3", "mir1", "MiR2", "MiR2", "miR4"),
                        Competing_expression = c(10000, 10000, 5000, 10000, 10000, 5000, 10000),
                        miRNA_expression = c(1000, 1000, 1000, 1000, 2000, 2000, 2000),
                        stringsAsFactors = FALSE)

testdata3 <- data.frame(competing = c("TP53", "Gene-2", "ABCC1", "Gene 4", "Gene 4", "Gene5", "PTEN"),
                        miRNA = c("mir1", "mir1","hsa-mir3", "mir1", "MiR2", "MiR2", "miR4"),
                        Competing_expression = c(NA, 10000, 5000, 10000, 10000, 5000, 10000),
                        miRNA_expression = c(1000, 1000, 1000, 1000, 2000, 2000, 2000),
                        stringsAsFactors = FALSE)

test_that("Can dataset be correctly converted to graph ? ", {


 prime_type_1 <-  (priming_graph(testdata, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
                      tidygraph::activate(nodes)%>%
      as_tibble()%>%
      filter(type == "miRNA")%>%
      count())%>%
   pull()

 prime_type_2 <-priming_graph(testdata, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
                   tidygraph::activate(nodes)%>%
     as_tibble()%>%
     filter(type == "Competing")%>%
     count()%>%
   pull()


 prime_type_3 <-  (priming_graph(testdata2, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
                      tidygraph::activate(nodes)%>%
                     as_tibble()%>%
                     filter(type == "miRNA")%>%
                     count())%>%
   pull()

 prime_type_4 <-priming_graph(testdata2, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
    tidygraph::activate(nodes)%>%
   as_tibble()%>%
   filter(type == "Competing")%>%
   count()%>%
   pull()




  expect_equal(prime_type_1 , prime_type_3)
  expect_equal(prime_type_2, prime_type_4)

})


test_that("Is there any missing value that is caused by calculations in graph?", {


  sum(is.na.data.frame(priming_graph(testdata, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
                         as_tibble()))->missings1

  sum(is.na.data.frame(priming_graph(testdata, competing_count = Competing_expression, miRNA_count= miRNA_expression)%>%
                          tidygraph::activate(nodes)%>%
                         as_tibble()))->missings2

  expect_equal(missings1, missings2)


}

)

test_that("When missing value is found in dataframe", {

   expect_warning(priming_graph(testdata3, competing_count = Competing_expression, miRNA_count= miRNA_expression), "Dataframe includes 1 NA values. Dataframe will be processed after NA removing. ")

          }

)

test_that("Simulation on the sample network", {

   data("minsamp")
   data("new_counts")

   minsamp %>%
      priming_graph(competing_count = Competing_expression,
                    miRNA_count = miRNA_expression,
                    aff_factor = c(energy, seed_type),
                    deg_factor = region) %>%
      update_variables(current_counts = new_counts) %>%
      vis_graph()->initialsim_res

   minsamp %>%
      priming_graph(competing_count = Competing_expression,
                    miRNA_count = miRNA_expression,
                    aff_factor = c(energy, seed_type),
                    deg_factor = region) %>%
      update_variables(current_counts = new_counts) %>%
      simulate(3) %>%
      vis_graph(title = "Minsamp Graph After 3 Iteration")->sim_res

   minsamp %>%
      priming_graph(competing_count = Competing_expression,
                    miRNA_count = miRNA_expression,
                    aff_factor = c(energy, seed_type),
                    deg_factor = region) %>%
      update_how("Gene2", how = 3) %>%
      simulate_vis(3, title = "Minsamp Graph After Each Iteration")%>%
      as_tibble()%>%
      filter(count_current == initial_count)%>%
      select(name)%>%
      pull()-> test_sim


   expect_equal(sim_res[["layers"]][[2]][["aes_params"]][["shape"]], 16)
   expect_equal(test_sim, c("Mir1", "Mir2"))


}

)

context("Check to Update")



test_that("Check the functions which update the values on the datasets.", {

  data("midsamp")
  data("minsamp")
  data("new_counts")

  primerg<- midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)


  seconderg<- primerg%>%
    update_how(sample_n(midsamp, 1)$Genes, 2)

  minsamp%>%
    priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)%>%
    update_variables(new_counts)%>%
    as_tibble()%>%
    select(count_current)%>%
    pull()-> min_res

  new_counts%>%
    mutate(Competing= ifelse(Competing== "Gene2", "Gene16", Competing))->new_counts2
  new_counts%>%
    mutate(miRNA= ifelse(miRNA== "Mir2", "Mir3", miRNA))-> new_counts3

  expect_equal(ncol(as_tibble(primerg%>%activate(nodes))), 7)
  expect_equal(ncol(as_tibble(seconderg%>%activate(nodes))), 7)
  expect_equal(ncol(as_tibble(seconderg%>%activate(edges))), ncol(as_tibble(primerg%>%activate(edges))))
  expect_equal(min_res, c(10000, 20000, 5000, 10000, 5000, 10000, 1000, 2000))
  expect_error(primerg%>%update_how("Gene30", 2), "Given node name Gene30 was not found! Please check it.")
  expect_error(primerg%>%update_how("Gene3", -2), "Fold change should not be less than zero.\n                     Please use decimal values for decrease. e.g. 0.5 for 2-fold decrease")
  expect_error(minsamp%>%
                 priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)%>%
                 update_variables(current_counts=new_counts2),
               "Current values include one or more different node name!")
  expect_error(minsamp%>%
                 priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)%>%
                 update_variables(current_counts=new_counts3),
               "Current values include one or more different node name!")
})


test_that("Check the functions which update the values on the datasets with interaction factors.", {

  data("midsamp")

  primergf<- midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)


  secondergf<- primergf%>%
    update_how(sample_n(midsamp, 1)$Genes, 2)


  midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    update_how("Gene15", 0.3)%>%
    simulate(10)%>%
    as_tibble()%>%
    dplyr::mutate(count_current= round(count_current, 2))%>%
    dplyr::select(count_current)%>%
    pull() -> outputall


  expect_equal(ncol(as_tibble(secondergf%>%activate(edges))), ncol(as_tibble(primergf%>%activate(edges))))
  expect_equal(outputall, c(10000, 10000,  5000,  9974,  5000,  9976,  6981,  2992,  5999,  2000,  7999,  1500,   500,  6983,  1021,  1995,  5984,  1000,  6500,  4800,  1000,  2000,  3000,  5000))
})

test_that("Check zero values", {

  data("midsamp")

  midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression,  aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene6", 0, knockdown = FALSE)%>%
    simulate(10, knockdown = FALSE)%>%
    as_tibble()%>%
    dplyr::mutate(count_current= round(count_current, 2))%>%
    dplyr::select(count_current)%>%
    pull() -> gene6_zero

  expect_equal(gene6_zero, c(10000, 10000,5000, 9952,5000, 708,6960, 3000,5970, 1995,7936, 1492, 498, 6752,2989, 1999,5949, 1000,6486, 4798,1000,2000, 3000,5000))


})



test_that("Check gene knock down", {

  data("midsamp")

  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = TRUE)%>%
    as_tibble()%>%
    filter( name == "Gene2")%>%
    select(count_current)%>%
    pull()-> res1


  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = TRUE)%>%
    simulate(cycle = 5, knockdown = TRUE)%>%
    as_tibble()%>%
    filter( name == "Gene2")%>%
    select(count_current)%>%
    pull()-> res2


  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = TRUE)%>%
    simulate(cycle = 5, knockdown = TRUE)%>%
    activate(edges)%>%
    as_tibble()%>%
    filter( Competing_name == "Gene2")%>%
    select(comp_count_pre)%>%
    pull()-> res3


  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = TRUE)%>%
    simulate(cycle = 5, knockdown = TRUE)%>%
    activate(edges)%>%
    as_tibble()%>%
    filter(Competing_name == "Gene2")%>%
    select(comp_count_current)%>%
    pull()-> res4

  expect_equal(res1, res2, 0)
  expect_equal(res3, res4, 0)


  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = TRUE)%>%
    simulate(cycle = 5, knockdown = TRUE)%>%
    activate(edges)%>%
    as_tibble()%>%
    filter( Competing_name == "Gene2")%>%
    select(comp_count_list)%>%
    unnest(comp_count_list)%>%
    pull()-> sim_res

  expect_equal(sim_res, c(10000, 0, 0, 0, 0, 0))

  midsamp %>%
    priming_graph(competing_count = Gene_expression,
                  miRNA_count = miRNA_expression, aff_factor = c(seeds, Energy), deg_factor = targeting_region)%>%
    update_how("Gene2", how=0, knockdown = FALSE)%>%
    simulate(cycle = 5, knockdown = FALSE)%>%
    as_tibble()%>%
    filter(name== "Gene2")%>%
    select(count_current)%>%
    round(digits = 2)%>%
    pull()->res5

  expect_equal(res5, 7)

})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message= FALSE, warning=FALSE--------------------------------------------
library(ceRNAnetsim)
library(png)

## ----message=FALSE, warning= FALSE, eval=FALSE--------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("ceRNAnetsim")

## ----fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'----
data("minsamp")

minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression) %>%
  update_how("Gene4",2) %>%
  simulate_vis(title = "Minsamp: Common element as trigger", cycle = 15)


minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression) %>%
  update_how("Gene4",2) %>%
  simulate(cycle = 5)

## -----------------------------------------------------------------------------
data("midsamp")

midsamp

## ----fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'----
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17",2) %>%
  simulate_vis(title = "Midsamp: Gene with higher degree as trigger", 15)

## -----------------------------------------------------------------------------
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene2",2) %>%
  simulate(50) %>% 
  find_iteration(limit=0)


## -----------------------------------------------------------------------------
data("midsamp_new_counts")

midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_variables(current_counts = midsamp_new_counts) %>%
  simulate(50) %>% 
  find_iteration(limit=0)

## ----fig.align='center', fig.width=5, fig.height=3, dpi= 120------------------
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17",2) %>%
  simulate(50) %>% 
  find_iteration(limit=0)


## ----fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'----
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17", 2) %>%
  simulate_vis(title = "Midsamp: Gene17 2 fold increase, 25 cycles", 25)

## ----fig.align='center', fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'----

midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17", 2) %>%
  simulate_vis(title = "Midsamp: Gene17 2 fold increase, 6 cycles", 6, threshold = 1)


## -----------------------------------------------------------------------------
midsamp %>%
  priming_graph(competing_count = Gene_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds), 
                deg_factor = targeting_region) %>%
  calc_perturbation("Gene17", 3, cycle = 30, limit = 0.1)

## -----------------------------------------------------------------------------
primed_mid <- midsamp %>%
  priming_graph(competing_count = Gene_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds), 
                deg_factor = targeting_region)

## ----eval=FALSE---------------------------------------------------------------
# # for parallel processing
# # future::plan(multiprocess)
# 
# seq(2,10) %>%
#   rlang::set_names() %>%
#   furrr::future_map_dfr(~ primed_mid %>% calc_perturbation("Gene17", .x, cycle = 30, limit = 0.1), .id="fold_change")

## ----warning=FALSE------------------------------------------------------------
midsamp %>%
  priming_graph(competing_count = Gene_expression,
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds),
                deg_factor = targeting_region) %>%
  find_node_perturbation(how = 3, cycle = 4, limit = 0.1)%>%
  select(name, perturbation_efficiency, perturbed_count)

## -----------------------------------------------------------------------------
midsamp %>%
  priming_graph(competing_count = Gene_expression,
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds),
                deg_factor = targeting_region) %>%
  find_node_perturbation(how = 3, cycle = 4, limit = 0.1, fast=5)%>%
  select(name, perturbation_efficiency, perturbed_count)

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

#' @title normalize
#'
#' @description normalizes the values according to maximum values
#' inside a group. The helper function of priming_graph.
#'
#' @param x The variable name that is normalized.
#'
#' @return normalized values
#' @keywords internal
#'


normalize <- function(x) {
  abs(x)/max(abs(x))
}

#' Converts the given dataframe using first variable as
#' competing and the second as miRNA.
#' The function converts the given dataframe using first
#' variable as competing and the second as miRNA. If user
#' defines interaction factors as affinity or degradation,
#' the factors are taken into account.
#'
#' @importFrom purrr reduce
#' @importFrom igraph edges edge
#' @importFrom stats na.omit
#' @importFrom rlang enquo enquos
#' @return the graph object.
#'
#' @param df A data frame that includes the miRNA and competing targets.
#' @param competing_count The counts (or expression) of competing elements of the dataset.
#' @param miRNA_count The counts (or expression) of repressive element (miRNA) of the dataset.
#' @param aff_factor The parameter/s of binding between miRNA and targets.
#' @param deg_factor The parameter/s for degradation of bound miRNA:target complex.
#'
#' @details priming_graph provides grouping of competing targets
#' and evaluation of targets within the groups taking into account
#' miRNA:target, target:total target, interaction and degradation
#' parameters. The target groups are determined according to miRNAs.
#' If the factors that are important in target interactions are
#' specified as arguments, the factors also are evaluated separately
#' within each group. priming_graph also calculates the miRNA efficiency
#' in steady-state conditions. It is assumed that quantity of competing
#' targets and miRNAs are shown in the steady-state system after the
#' miRNAs exhibit repressive efficiency. Note that the data must not
#' include missing values such as NA or '-'.
#'
#'
#' @examples
#'
#' data('minsamp')
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression)
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression,
#'     aff_factor = c(seed_type,energy), deg_factor = region)
#'
#'
#' @export

priming_graph <- function(df, competing_count, miRNA_count, aff_factor = dummy,
                          deg_factor = dummy) {

  if (any(is.na(df))) {

    warning(paste("Dataframe includes", sum(is.na(df)), "NA values. Dataframe will be processed after NA removing. "))
    df <- na.omit(df)

  }

  competing_exp <- enquo(competing_count)
  mirna_exp <- enquo(miRNA_count)
  affinity <- enquos(aff_factor)
  degradation <- enquos(deg_factor)

  df <- df %>% dplyr::mutate(competing = .[[1]], miRNA = .[[2]], Competing_name = .[[1]],
                             miRNA_name = .[[2]], dummy = 1) %>% dplyr::select(competing, miRNA,
                                                                               Competing_name, miRNA_name, !!competing_exp, !!mirna_exp, !!!affinity,
                                                                               !!!degradation, dummy)


  input_graph <- df %>% dplyr::group_by(miRNA) %>% dplyr::mutate_at(dplyr::vars(!!!affinity),
                                                                    list(anorm = ~normalize(.))) %>% 
                                                   dplyr::mutate_at(dplyr::vars(!!!degradation),
                                                                    list(dnorm = ~normalize(.))) %>% 
                                                   dplyr::ungroup() %>% 
    dplyr::mutate(afff_factor = dplyr::select(., dplyr::ends_with("anorm")) %>% 
                    reduce(`*`, .init = 1), degg_factor = dplyr::select(., dplyr::ends_with("dnorm")) %>% 
                    reduce(`*`, .init = 1)) %>% 
    tidygraph::as_tbl_graph() %>%
    tidygraph::activate(nodes) %>% tidygraph::mutate(type = ifelse(tidygraph::centrality_degree(mode = "in") >
                                                                     0, "miRNA", "Competing")) %>% tidygraph::mutate(node_id = dplyr::row_number()) %>%
    tidygraph::activate(edges) %>% tidygraph::mutate(comp_count_list = as.list(!!competing_exp),
                                                     comp_count_pre = !!competing_exp, comp_count_current = !!competing_exp,
                                                     mirna_count_list = as.list(!!mirna_exp), mirna_count_pre = !!mirna_exp,
                                                     mirna_count_current = !!mirna_exp) %>% tidygraph::group_by(to) %>%
    tidygraph::mutate(mirna_count_per_dep = mirna_count_current * comp_count_current *afff_factor/sum(comp_count_current * afff_factor), 
                      mirna_count_per_dep = ifelse(is.na(mirna_count_per_dep),0, mirna_count_per_dep)) %>% 
    tidygraph::ungroup() %>% tidygraph::mutate(effect_current = mirna_count_per_dep * degg_factor, 
                                               effect_pre = effect_current, 
                                               effect_list = as.list(effect_current)) %>%
    tidygraph::select(-dplyr::ends_with("norm"), dummy)

  warning("First column is processed as competing and the second as miRNA.
")


  return(input_graph %>% update_nodes(once = TRUE))

}

context("Most significant node detection")

test_that("Could the function detect most affected nodes from a specific node in ceRNA network? ",{
  
  data("midsamp")

  midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    find_affected_nodes(node_name = "Gene1", how = 2, cycle = 2, top = 2)->test_affected_nodes
  
  midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    find_affected_nodes(node_name = "Gene1", how = 2, cycle = 2)->test_affected_nodes_2
  
  expect_true(is.data.frame(test_affected_nodes_2))
  expect_true(is.data.frame(test_affected_nodes))
  expect_equal(test_affected_nodes$top_nodes, c("Gene2", "Gene4" ))
  expect_equal(test_affected_nodes$change[1], "upregulated")
  expect_equal(length(test_affected_nodes_2$change), 5)

})





test_that("Could the function detect potential effective node(s) on a specified node? ",{
  
  data("midsamp")
  
  midsamp%>%
    priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)%>%
    find_targeting_nodes(how = 2, cycle = 2, target = "Gene1")->test_targeting
  

  
  expect_true(is.data.frame(test_targeting))
  expect_equal(test_targeting$targeting_node, c("Gene2", "Gene3", "Gene4", "Mir1"))
  expect_equal(test_targeting$change[4], "downregulated")

  
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message= FALSE, warning=FALSE--------------------------------------------
library(ceRNAnetsim)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("ceRNAnetsim")

## ----warning= FALSE, message=FALSE--------------------------------------------
data("mirtarbasegene")
head(mirtarbasegene)

## -----------------------------------------------------------------------------
data("TCGA_E9_A1N5_normal")
head(TCGA_E9_A1N5_normal)

## -----------------------------------------------------------------------------
data("TCGA_E9_A1N5_tumor")
head(TCGA_E9_A1N5_tumor)

## -----------------------------------------------------------------------------
data("TCGA_E9_A1N5_mirnatumor")
head(TCGA_E9_A1N5_mirnatumor)

## -----------------------------------------------------------------------------
data("TCGA_E9_A1N5_mirnanormal")
head(TCGA_E9_A1N5_mirnanormal)

## -----------------------------------------------------------------------------
TCGA_E9_A1N5_mirnagene <- TCGA_E9_A1N5_mirnanormal %>%
  inner_join(mirtarbasegene, by= "miRNA") %>%
  inner_join(TCGA_E9_A1N5_normal, 
             by = c("Target"= "external_gene_name")) %>%
  select(Target, miRNA, total_read, gene_expression) %>%
  distinct() 

## ----echo=FALSE---------------------------------------------------------------

TCGA_E9_A1N5_mirnagene%>%
  group_by(Target, miRNA)%>%
  count()%>%
  filter(n==2)

TCGA_E9_A1N5_mirnagene %>%    
  group_by(Target) %>%        
  mutate(gene_expression= max(gene_expression)) %>%
  distinct() %>%
  ungroup() -> TCGA_E9_A1N5_mirnagene


## -----------------------------------------------------------------------------
head(TCGA_E9_A1N5_mirnagene)

## -----------------------------------------------------------------------------
TCGA_E9_A1N5_mirnagene <- TCGA_E9_A1N5_mirnagene%>%
  filter(gene_expression > 10)

## ----warning=FALSE, fig.height=5, fig.width=6, fig.align='center', warning=FALSE----

simulation_res_HIST <- TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read) %>% 
  update_how(node_name = "HIST1H3H", how =30) %>% 
  simulate(5)

simulation_res_HIST%>%
  find_iteration(plot=TRUE)


## -----------------------------------------------------------------------------

simulation_res_HIST%>%
  as_tibble()%>%
  mutate(FC= count_current/initial_count)%>%
  arrange(desc(FC))

## ----warning=FALSE, fig.height=5, fig.width=6, fig.align='center', warning=FALSE----
simulation_res_ACTB <- TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read) %>% 
  update_how(node_name = "ACTB", how =1.87) %>% 
  simulate(5)

simulation_res_ACTB%>%
  find_iteration(plot=TRUE)

## -----------------------------------------------------------------------------

simulation_res_ACTB%>%
  as_tibble()%>%
  mutate(FC= count_current/initial_count)%>%
  arrange(desc(FC))

## ----message=FALSE, warning=FALSE---------------------------------------------
data("huge_example")
head(huge_example)

## -----------------------------------------------------------------------------
filtered_example <- huge_example %>%
  add_count(competing) %>%
  filter(n > 5) %>%
  select(-n)

head(filtered_example)

## ----fig.height=5, fig.width=6, fig.align='center', warning=FALSE-------------
simulation_GAPDH <- filtered_example %>%
  priming_graph(competing_count = competing_counts, 
                miRNA_count = mirnaexpression_normal, 
                aff_factor = Energy) %>%
  update_how("GAPDH", 5) 

simulation_GAPDH%>%
  vis_graph(title = "Distribution of GAPDH gene node")

## ----fig.height=5, fig.width=6, fig.align='center', warning=FALSE, fig.show='hide'----

simulation_GAPDH%>%
  simulate_vis(title = "GAPDH over expression in the real dataset", 3)

## ----warning=FALSE------------------------------------------------------------

entire_perturbation <- filtered_example%>%
  priming_graph(competing_count = competing_counts, miRNA_count = mirnaexpression_normal)%>%
  find_node_perturbation(how=5, cycle=3, fast = 15)%>%
  select(name, perturbation_efficiency, perturbed_count)
  

## ----warning=FALSE------------------------------------------------------------

entire_perturbation%>%
  filter(!is.na(perturbation_efficiency), !is.na(perturbed_count))%>%
  select(name, perturbation_efficiency, perturbed_count)

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

#' Finds top affected nodes for perturbation 
#' from a particular node
#'
#'
#' @importFrom tidygraph as_tibble
#' @importFrom utils head
#'
#' @return It gives a tibble form dataset that includes perturbation node, 
#' affected nodes and changes of them.
#'
#' @details Lists the most affected nodes after perturbation 
#' initiated from a particular node. In the background, 
#' it compares the calculated values after
#' the simulation with their initial values.
#'
#' @param input_graph The graph object that was processed with priming_graph function.
#' @param node_name The node to trigger perturbations.
#' @param how The change of count (expression) of the given node in terms of fold change.
#' @param cycle The iteration of simulation.
#' @param limit The minimum fold change which can be taken into account for
#' perturbation calculation on all nodes in terms of percentage.
#' @param top Determines how many nodes most affected will be listed.
#'
#' @examples
#'
#' data('midsamp')
#' 
#' midsamp%>%
#' priming_graph(competing_count = Gene_expression, 
#'               miRNA_count = miRNA_expression)%>%
#' find_affected_nodes(node_name = "Gene1", 
#'                     how = 2, 
#'                     cycle = 2, 
#'                     top = 2)
#'
#'
#' @export



find_affected_nodes<- function(input_graph, node_name, how = 1, cycle = 1,
                               limit = 0, top = 5) {

  res <- input_graph %>% 
    update_how(node_name, how) %>% 
    simulate(cycle) %>%
    tidygraph::as_tibble() %>% 
    dplyr::filter(name != node_name)  
  
  top_nodes <- res%>%
    arrange(desc(abs(count_current - initial_count)))%>%
    head(top)%>%
    select(name)%>%
    as.list()
  
  initial_exp <- res%>%
    arrange(desc(abs(count_current - initial_count)))%>%
    head(top)%>%
    select(initial_count)%>%
    as.list()
  
  final_exp <- res%>%
    arrange(desc(abs(count_current - initial_count)))%>%
    head(top)%>%
    select(count_current)%>%
    as.list()
  
  top_changed <- res%>%
    arrange(desc(abs(count_current - initial_count)))%>%
    mutate(change = ifelse(count_current - initial_count < 0, "downregulated", "upregulated"),
           change = ifelse(count_current - initial_count == 0, "Not changed", change))%>%
    head(top)%>%
    select(change)%>%
    as.list()
  
  
  tibble(source = node_name, top_nodes = top_nodes, change = top_changed, initial_exp = initial_exp, final_exp=final_exp)%>%
    unnest(cols = c(source, top_nodes, change, initial_exp, final_exp))->res
  
  return(res)
  
}

#' @name huge_example
#' @title huge example
#' @description A sample dataset which is utilised through integration of TCGA_E9_A1N5_normal,
#'     TCGA_E9_A1N5_mirnanormal and high-throughput experimental miRNA:gene dataset.
#' @docType data
#' @format A data frame with 7 variables and 26176 observation:
#' \describe{
#'   \item{competing}{name of gene}
#'   \item{miRNA}{name of miRNA}
#'   \item{competing_counts}{Expression values of competing element (gene)}
#'   \item{mirnaexpression_normal}{Expression value of miRNA elements in normal tissue}
#'   \item{Energy}{Energy of miRNA:target binding}
#'   \item{region_effect}{Coefficient for efficiency of location on target}
#'   \item{seed_type_effect}{Coefficient for efficiency of seed sequence of miRNA:target interaction}
#'   }
#' @source Dataset was integrated by us.
#'

NULL


#' @name midsamp
#' @title midsamp
#' @description middle sized sample dataset
#' @docType data
#' @format A data frame with 7 variables and 26 observation of them:
#' \describe{
#'   \item{Genes}{symbol of gene}
#'   \item{miRNAs}{symol of miRNA}
#'   \item{Gene_expression}{Expression values of competing gene}
#'   \item{miRNA_expression}{Expression value of miRNA}
#'   \item{seeds}{Coefficient for efficiency of seed type of miRNA:target interaction}
#'   \item{targeting_region}{Coefficient for efficiency of location on target}
#'   \item{Energy}{Energy of miRNA:target binding}
#'   }
#'
#' @source Dataset was created by us.
#'

NULL

#' @name midsamp_new_counts
#' @title midsamp_new_counts
#' @description includes new expression values for middle sized sample dataset
#' @docType data
#' @format A data frame with 4 variables and 26 observation of them:
#' \describe{
#'   \item{Competing}{symbol of gene}
#'   \item{miRNA}{symol of miRNA}
#'   \item{Competing_count}{Expression values of competing gene}
#'   \item{miRNA_count}{Expression value of miRNA}
#'   }
#' @source Dataset was created by us.
#'

NULL

#' @name minsamp
#' @title minsamp
#' @description minimal sample dataset
#' @docType data
#' @format A data frame with 7 variables and 7 observation of them:
#' \describe{
#'   \item{competing}{symbol of gene}
#'   \item{miRNA}{symol of miRNA}
#'   \item{Competing_expression}{Expression values of competing gene}
#'   \item{miRNA_expression}{Expression value of miRNA}
#'   \item{seed_type}{Coefficient for efficiency of seed sequence of miRNA:target interaction}
#'   \item{region}{Coefficient for efficiency of location on target}
#'   \item{energy}{Energy of miRNA:target binding}
#'   }
#' @source Dataset was created by us.
#'

NULL

#' @name new_counts
#' @title new_counts
#' @description includes new expression values for minimal sample dataset
#' @docType data
#' @format A data frame with 7 variables and 7 observation of them:
#' \describe{
#'   \item{Competing}{symbol of gene}
#'   \item{miRNA}{symol of miRNA}
#'   \item{Competing_count}{Expression values of competing gene}
#'   \item{miRNA_count}{Expression value of miRNA}
#'   }
#'
#' @source Dataset was created by us.
#'

NULL

#' @name mirtarbasegene
#' @title mirtarbasegene
#' @description the dataset that includes miRNA:target gene interactions downloaded from mirtarbase
#' @source \url{http://mirtarbase.mbc.nctu.edu.tw/php/index.php}
#' @format Classes tbl_df, tbl and data.frame with 380627 observation of 2 variables:
#' \describe{
#'   \item{miRNA}{miRNA symbol}
#'   \item{Target}{target gene symbol}
#'   }
#' @docType data

NULL

#' @name TCGA_E9_A1N5_mirnanormal
#' @title TCGA_E9_A1N5_mirnanormal
#' @description The dataset contains mirna expression values for normal tissue sample of TCGA-E9-A1N5 barcoded patient
#' @source \url{https://portal.gdc.cancer.gov/}
#' @format Classes tbl_df, tbl and data.frame with 750 observation of 6 variables:
#' \describe{
#'   \item{barcode}{Sample, normal tissue, barcode of patient based on TCGA}
#'   \item{mirbase_ID}{mirbase id of miRNA}
#'   \item{miRNA}{miRNA name}
#'   \item{Precusor}{Precusor id of miRNA which is given in miRNA variable}
#'   \item{total_read}{total reading count of miRNA which is produced from different gene locations}
#'   \item{total_RPM}{total RPM (reading per million) of miRNA}
#'   }
#'
#' @docType data

NULL

#' @name TCGA_E9_A1N5_mirnatumor
#' @title TCGA_E9_A1N5_mirnatumor
#' @description The dataset contains mirna expression values for tumor tissue sample of TCGA-E9-A1N5 barcoded patient
#' @source \url{https://portal.gdc.cancer.gov/}
#' @format Classes tbl_df, tbl and data.frame with 648 observation of 6 variables:
#' \describe{
#'   \item{barcode}{Sample, tumor tissue, barcode of patient based on TCGA}
#'   \item{mirbase_ID}{mirbase id of miRNA}
#'   \item{miRNA}{miRNA name}
#'   \item{Precusor}{Precusor id of miRNA which is given in miRNA variable}
#'   \item{total_read}{total reading count of miRNA which is produced from different gene locations}
#'   \item{total_RPM}{total RPM (reading per million) of miRNA}
#'   }
#' @docType data

NULL

#' @name TCGA_E9_A1N5_normal
#' @title TCGA_E9_A1N5_normal
#' @description The dataset contains gene expression values for normal tissue sample of TCGA-E9-A1N5 barcoded patient
#' @source \url{https://portal.gdc.cancer.gov/}
#' @format Classes tbl_df, tbl and data.frame with 56830 observation of 7 variables:
#' \describe{
#'   \item{patient}{Barcode of patient based on TCGA}
#'   \item{sample}{Tissue sample barcode of the patient}
#'   \item{barcode}{Sample barcode of the patient}
#'   \item{definition}{Tissue type of sample (Solid Tissue Normal)}
#'   \item{ensembl_gene_id}{Gene id}
#'   \item{external_gene_name}{Gene symbol}
#'   \item{gene_expression}{Gene expression value}
#'   }
#' @docType data


NULL

#' @name TCGA_E9_A1N5_tumor
#' @title TCGA_E9_A1N5_tumor
#' @description The dataset contains gene expression values for cancer tissue sample of TCGA-E9-A1N5 barcoded patient
#' @source \url{https://portal.gdc.cancer.gov/}
#' @format Classes tbl_df, tbl and data.frame with 56830 observtion of 7 variables:
#' \describe{
#'   \item{patient}{Barcode of patient based on TCGA}
#'   \item{sample}{Tissue sample barcode of the patient}
#'   \item{barcode}{Sample barcode of the patient}
#'   \item{definition}{Tissue type of sample (Primary solid Tumor)}
#'   \item{ensembl_gene_id}{Gene id}
#'   \item{external_gene_name}{Gene symbol}
#'   \item{gene_expression}{Gene expression value}
#'   }
#' @docType data

NULL

#' Utilizes the change in expression value/s as triggering.
#'
#' simulate function uses the change in expression value/s as triggering.
#' @import dplyr tidygraph
#' @importFrom purrr pmap
#' @return The graph.
#'
#' @details The steady-state conditions of the system are disturbed
#' after the change in the graph (with update_how or update_variables).
#' In this case, the system tend to be steady state again. The arrangement
#' of competetive profiles of the targets continue until all nodes are
#' updated and steady-state nearly. Note that, If `how` argument is
#' specified as `0`, *simulate()* and *update_how()* functions process
#' the variables to knockdown of specified gene with default `knockdown = TRUE`
#' and knocked down competing RNA is kept at zero. However, if `knockdown= FALSE`
#' argument is applied, competing RNA which has initial expression level of zero
#' is allowed to increase or fluctuate during calculations.
#'
#' @param input_graph The graph object that processed in previous steps.
#' @param cycle Optimal iteration number for gaining steady-state.
#' @param threshold absolute minimum amount of change required to be
#' considered as up/down regulated element
#' @param knockdown specifies gene knockdown with default TRUE
#'
#' @examples
#'
#' data('minsamp')
#' data('new_counts')
#'
#' ## new_counts, the dataset that includes the current counts of nodes.
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression)%>%
#'   update_variables(new_counts)%>%
#'   simulate()
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression,
#'        aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'   update_variables(new_counts)%>%
#'   simulate(cycle = 3)
#'
#' @export

simulate <- function(input_graph, cycle = 1, threshold = 0, knockdown=TRUE) {
  
  
  if (cycle < 1)
    stop("number of cycles should be more than 1", call. = FALSE)
  
  if (knockdown) {
    
    for (i in seq_len(cycle)) {
      
      input_graph <- input_graph %>% 
        tidygraph::activate(edges) %>%
        tidygraph::group_by(to) %>% 
        tidygraph::mutate(mirna_count_per_comp = mirna_count_current *comp_count_current * afff_factor/sum(comp_count_current *afff_factor),
                          mirna_count_per_comp = ifelse(is.na(mirna_count_per_comp),0, mirna_count_per_comp)) %>% 
        tidygraph::ungroup() %>%
        tidygraph::mutate(effect_pre = effect_current, 
                          effect_current = mirna_count_per_comp *degg_factor) %>% 
        tidygraph::group_by(from) %>%
        # NEW
        tidygraph::mutate(avg_count_current = round((comp_count_pre+comp_count_current)/2, 0)) %>% 
        tidygraph::mutate(comp_count_pre = ifelse(comp_count_current < 0, 1, comp_count_current), 
                          comp_count_current = ifelse(comp_count_pre ==0, 0, comp_count_pre - (sum(effect_current) - sum(effect_pre))),
                          comp_count_current = ifelse(comp_count_current < 0, 1, comp_count_current)) %>% 
        # NEW 
        tidygraph::mutate(avg_count_pre = avg_count_current, 
                          avg_count_current = round((comp_count_pre+comp_count_current)/2, 0)) %>% 
        tidygraph::ungroup() %>% 
        # NEW: this fixes the ossilation issue:
        mutate(comp_count_current = ifelse(abs(avg_count_pre - avg_count_current) < 0.001*avg_count_current, avg_count_current, comp_count_current)) %>% 
        #mutate(comp_count_current = ifelse(avg_count_pre == avg_count_current, avg_count_current, comp_count_current)) %>% 
        tidygraph::mutate(comp_count_list = pmap(list(comp_count_list,comp_count_current), c), 
                          effect_list = pmap(list(effect_list, effect_current), c),
                          mirna_count_list = pmap(list(mirna_count_list, mirna_count_current), c)) %>% 
        tidygraph::activate(nodes) %>%
        update_nodes(limit = threshold)
    }
    return(input_graph %>% activate(edges) %>% select(-avg_count_current, -avg_count_pre) %>% activate(nodes))
    
  }
  
  for (i in seq_len(cycle)) {
    
    input_graph <- input_graph %>% 
      tidygraph::activate(edges) %>% 
      tidygraph::group_by(to) %>%
      tidygraph::mutate(mirna_count_per_comp = mirna_count_current * comp_count_current * afff_factor/sum(comp_count_current * afff_factor), 
                        mirna_count_per_comp = ifelse(is.na(mirna_count_per_comp), 0, mirna_count_per_comp)) %>% 
      tidygraph::ungroup() %>%
      tidygraph::mutate(effect_pre = effect_current, effect_current = mirna_count_per_comp *degg_factor) %>% 
      tidygraph::group_by(from) %>% 
      # NEW
      tidygraph::mutate(avg_count_current = round((comp_count_pre+comp_count_current)/2, 0)) %>% 
      tidygraph::mutate(comp_count_pre = ifelse(comp_count_current <0, 1, comp_count_current), 
                        comp_count_current = comp_count_pre -(sum(effect_current) - sum(effect_pre)), 
                        comp_count_current = ifelse(comp_count_current <0, 1, comp_count_current)) %>% 
      # NEW 
      tidygraph::mutate(avg_count_pre = avg_count_current, 
                        avg_count_current = round((comp_count_pre+comp_count_current)/2, 0)) %>% 
      tidygraph::ungroup() %>% 
      # NEW: this fixes the ossilation issue:
      mutate(comp_count_current = ifelse(abs(avg_count_pre - avg_count_current) < 0.001*avg_count_current, avg_count_current, comp_count_current)) %>% 
      #mutate(comp_count_current = ifelse(avg_count_pre == avg_count_current, avg_count_current, comp_count_current)) %>% 
      tidygraph::mutate(comp_count_list = pmap(list(comp_count_list,comp_count_current), c), 
                        effect_list = pmap(list(effect_list, effect_current), c),
                        mirna_count_list = pmap(list(mirna_count_list, mirna_count_current), c)) %>% 
      tidygraph::activate(nodes) %>%
      update_nodes(limit = threshold)
  }
  return(input_graph %>% activate(edges) %>% select(-avg_count_current, -avg_count_pre) %>% activate(nodes))
}

#' Calculates average expression changes of all nodes except
#' trigger and finds the perturbed node count for a given node.
#'
#' @importFrom tibble tibble
#'
#' @details calc_perturbation calculates mean expression changes
#' of elements except trigger after the change in the network
#' in terms of percentage. It also calculates the number of nodes
#' that have expression changes after the change occur in the network.
#' The function determines the perturbation efficiency and number
#' of perturbed nodes after given change with how, cycle and
#' limit parameter.
#'
#' @param input_graph the graph object that was processed with
#' priming graph in previous step.
#' @param node_name The node that is trigger for simulation.
#' @param how The change of count of the given node in terms of fold change.
#' @param cycle The iteration of simulation.
#' @param limit The minimum fold change which can be taken into
#' account for perturbation calculation on all nodes in terms of percentage.
#'
#' @return a tibble with two columns, the perturbation
#' efficiency and number of perturbed nodes.
#'
#' @examples
#'
#' data('minsamp')
#'
#' minsamp%>%
#'    priming_graph(competing_count = Competing_expression,
#'        miRNA_count = miRNA_expression)%>%
#'    calc_perturbation('Gene6', how= 3, cycle = 4)
#'
#'  minsamp%>%
#'    priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression,
#'        aff_factor = c(energy,seed_type), deg_factor = region)%>%
#'    calc_perturbation('Gene6',3, cycle = 4)
#'
#'
#' @export


calc_perturbation <- function(input_graph, node_name, how = 1, cycle = 1,
                              limit = 0) {

  res <- input_graph %>% update_how(node_name, how) %>% simulate(cycle) %>%
    tidygraph::as_tibble() %>% dplyr::filter(name != node_name)

  perturbation_eff <- as.double((res %>% summarise(mean(abs(count_current -
                                                              initial_count) * 100/initial_count)))[[1]])

  perturbed_count <- as.double((res %>% filter((abs(count_current - initial_count) *
                                                  100/initial_count) > limit) %>% count())[[1]])

  return(tibble(perturbation_efficiency = perturbation_eff, perturbed_count = perturbed_count))
}

#' Carries the variables from edge to node.
#'
#' Carries the variables from edge to node.
#'
#' @details The function is a helper function for
#' processing of graph object with update_nodes function.
#'
#' @param input_graph Processed graph object in previous step.
#' @return tibble object
#' @keywords internal
#'


prepare_rhs_once <- function(input_graph) {

  dplyr::bind_rows((tidygraph::as_tibble(input_graph %>% tidygraph::activate(edges) %>%
                                           tidygraph::select(Competing_name, comp_count_current, comp_count_pre)) %>%
                      tidygraph::mutate(initial_count = comp_count_pre) %>% tidygraph::select(id = Competing_name,
                                                                                              initial_count, count_pre = comp_count_pre, count_current = comp_count_current) %>%
                      tidygraph::distinct()), (tidygraph::as_tibble(input_graph %>% tidygraph::activate(edges) %>%
                                                                      tidygraph::select(miRNA_name, mirna_count_current, mirna_count_pre)) %>%
                                                 tidygraph::mutate(initial_count = mirna_count_pre) %>% tidygraph::select(id = miRNA_name,
                                                                                                                          initial_count, count_pre = mirna_count_pre, count_current = mirna_count_current) %>%
                                                 tidygraph::distinct()))


}

#' Carries the variables from edge to node
#'
#' Carries the variables from edge to node.
#'
#' @details The function is a helper function for
#' processing of graph object with update_nodes function.
#'
#' @param input_graph Processed graph object in previous step.
#' @return tibble object
#' @keywords internal
#'


prepare_rhs <- function(input_graph) {

  dplyr::bind_rows((tidygraph::as_tibble(input_graph %>% tidygraph::activate(edges)) %>%
                      dplyr::select(id = Competing_name, count_current = comp_count_current) %>%
                      dplyr::distinct()), (tidygraph::as_tibble(input_graph %>% tidygraph::activate(edges)) %>%
                                             dplyr::select(id = miRNA_name, count_current = mirna_count_current) %>%
                                             dplyr::distinct()))
}

#' Carries variables from edge to node.
#'
#' This function carries variables from edge to node and should be used
#' after `update_how` or `update_variables` functions
#'
#' @return the graph object.
#'
#' @param input_graph Processed graph object in previous step.
#' @param once The argument is about when the carrying process
#' runs (internal use only)
#' @param limit absolute minimum amount of change required to
#' be considered as up/down regulated element
#'
#' @details If the carrying process performs after priming_graph
#' function, the argument must be TRUE. The function helps to
#' visualisation of processed graph object, especially that
#' includes too many nodes.This step makes it easily to follow
#' the processes.
#'
#' @examples
#'
#' data('minsamp')
#'
#' minsamp %>%
#'   priming_graph(Competing_expression, miRNA_expression) %>%
#'   update_how('Gene2',2)
#'
#' @export

update_nodes <- function(input_graph, once = FALSE, limit = 0) {
  if (once) {
    rhs_table <- prepare_rhs_once(input_graph)
  } else {
    rhs_table <- prepare_rhs(input_graph)
  }
  if (!once) {
    input_graph <- input_graph %>% tidygraph::activate(nodes) %>% tidygraph::select(-count_pre) %>%
      tidygraph::select(name, type, node_id, initial_count, count_pre = count_current)
  }

  input_graph <- input_graph %>% tidygraph::activate(nodes) %>% tidygraph::left_join(rhs_table,
                                                                                     by = c(name = "id")) %>% tidygraph::mutate(changes_variable = ifelse(count_current -
                                                                                                                                                            count_pre < -limit, "Down", type), changes_variable = ifelse(count_current -
                                                                                                                                                                                                                           count_pre > limit, "Up", changes_variable))

  input_graph
}

#' Provides visualisation of the graph.
#'
#' `vis_graph` Provides visualisation of the graph.
#'
#' @importFrom ggraph ggraph geom_edge_link geom_node_point
#' create_layout theme_graph
#' @importFrom ggplot2 scale_colour_manual ggtitle
#' @importFrom igraph V 'V<-'
#' @return The graph object.
#'
#' @details vis_graph ensures the process to be followed.
#'
#' @param input_graph The graph object.
#' @param Competing_color The color of competing elements
#' on the graph with 'green' default.
#' @param mirna_color The color of miRNAs on the graph with
#' 'orange' default.
#' @param Upregulation The color of Upregulated elements on
#' the graph with 'red' default.
#' @param Downregulation The color of Downregulated elements
#' on the graph with 'blue' default.
#' @param title Title of the given graph.
#' @param layout The layout that will be used for
#' visualisation of the graph.
#'
#' @examples
#'
#' data('minsamp')
#' data('new_counts')
#'
#' # Visualisation of graph in steady-state.
#'
#'  priming_graph(minsamp, Competing_expression, miRNA_expression,
#'     aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'    vis_graph()
#'
#' # Visualisation of graph after the change.
#'
#'  priming_graph(minsamp, Competing_expression, miRNA_expression,
#'     aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'    update_variables(new_counts)%>%
#'    vis_graph()
#'
#' @export

vis_graph <- function(input_graph, Competing_color = "green", mirna_color = "orange",
                      Upregulation = "red", Downregulation = "blue", title = "GRAPH", layout = "kk") {

  if (missing(layout)) {
    warning("Kamada-Kawai (kk) layout was used.")
  }

  input_graph%>% ggraph(layout = layout) + 
                  geom_edge_link(colour = "#838B8B", alpha = 0.5) + 
                  geom_node_point(aes(filter = (changes_variable == "Competing"), color = "Competing", size = count_current), shape = 16) +
                  geom_node_point(aes(filter = (changes_variable == "miRNA"), color = "miRNA"),shape = 16, size = 2) + 
                  geom_node_point(aes(filter = (changes_variable =="Up"), color = "Up", size = count_current), shape = 16) + 
                  geom_node_point(aes(filter = (changes_variable == "Down"), color = "Down", size = count_current), shape = 16) + 
                  scale_colour_manual(name = "Types", values = c(Competing = Competing_color, miRNA = mirna_color, Up = Upregulation, Down = Downregulation)) + 
                  ggtitle(title) + theme_graph(base_family = "sans")


}

#'  Finds the iteration which provides maximum affected node number
#'
#'  searches the iteration that provides maximum affected node number.
#'  The user defines a symbolic iteration with .iter. The function
#'  calculates the number of affected nodes for each iteration and
#'  then selects the iteration that has maximum affected nodes' number.
#'
#' @importFrom ggplot2 ggplot xlab geom_line ylab aes
#' @importFrom purrr map_dbl
#' @return It gives an iteration number to use in simulate() function.
#'
#' @param df A tbl graph that includes the miRNA and competing targets
#' triggered and simulated for number of cycles.
#' @param limit The minimum amount of change of any node.
#' @param plot If TRUE, returns a plot.
#'
#' @examples
#'
#' data('midsamp')
#'
#' midsamp %>%
#'   priming_graph(Gene_expression, miRNA_expression) %>%
#'   update_how('Gene2',2) %>%
#'   simulate(10) %>%
#'   find_iteration(limit=0)
#'
#' @export

find_iteration <- function(df, limit = 0.1, plot = FALSE) {

  len <- df %>% tidygraph::activate(edges) %>% tidygraph::as_tibble() %>%
    .$comp_count_list %>% .[[1]] %>% length()
  iteration <- data.frame(iter = seq(1, len - 1, 1), effect = rep(0))

  for (i in seq_len((len - 1))) {

    iteration$effect[i] <- df %>% tidygraph::activate(edges) %>% tidygraph::as_tibble() %>%
      dplyr::select(from, comp_count_list) %>% dplyr::mutate(dif = (abs(map_dbl(comp_count_list,
                                                                                i + 1)) - abs(map_dbl(comp_count_list, i)))) %>% dplyr::select(-comp_count_list) %>%
      dplyr::distinct() %>% dplyr::mutate(non_zero = ifelse(abs(dif) >
                                                              limit, 1, 0)) %>% dplyr::summarise(per = sum(non_zero) * 100/n()) %>%
      pull()

  }

  if (plot) {
    return(iteration %>% ggplot(aes(y = effect, x = iter)) + geom_line() +
             xlab("Iteration") + ylab("(%) The Disturbed Element"))

  } else {
    return((iteration %>% dplyr::filter(effect == max(effect)))[1,
                                                                1])
  }
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, warning= FALSE, message=FALSE-------------------------------------
library(ceRNAnetsim)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("ceRNAnetsim")

## -----------------------------------------------------------------------------
data("TCGA_E9_A1N5_tumor")
data("TCGA_E9_A1N5_normal")
data("mirtarbasegene")
data("TCGA_E9_A1N5_mirnanormal")

## -----------------------------------------------------------------------------
TCGA_E9_A1N5_mirnanormal %>%
  inner_join(mirtarbasegene, by= "miRNA") %>%
  inner_join(TCGA_E9_A1N5_normal, 
             by = c("Target"= "external_gene_name")) %>%
  select(Target, miRNA, total_read, gene_expression) %>%
  distinct() -> TCGA_E9_A1N5_mirnagene

## -----------------------------------------------------------------------------
TCGA_E9_A1N5_tumor%>%
  inner_join(TCGA_E9_A1N5_normal, by= "external_gene_name")%>%
  select(patient = patient.x, 
         external_gene_name, 
         tumor_exp = gene_expression.x, 
         normal_exp = gene_expression.y)%>%
  distinct()%>%
  inner_join(TCGA_E9_A1N5_mirnagene, by = c("external_gene_name"= "Target"))%>%
  filter(tumor_exp != 0, normal_exp != 0)%>%
  mutate(FC= tumor_exp/normal_exp)%>%
  filter(external_gene_name== "HIST1H3H")

#HIST1H3H: interacts with various miRNA in dataset, so we can say that HIST1H3H is non-isolated competing element and increases to 30-fold.


## -----------------------------------------------------------------------------
TCGA_E9_A1N5_tumor%>%
  inner_join(TCGA_E9_A1N5_normal, by= "external_gene_name") %>%
  select(patient = patient.x, 
         external_gene_name, 
         tumor_exp = gene_expression.x, 
         normal_exp = gene_expression.y) %>%
  distinct() %>%
  inner_join(TCGA_E9_A1N5_mirnagene, 
             by = c("external_gene_name"= "Target")) %>%
  filter(tumor_exp != 0, normal_exp != 0) %>%
  mutate(FC= tumor_exp/normal_exp) %>%
  filter(external_gene_name == "ACTB")

#ACTB: interacts with various miRNA in dataset, so ACTB is not isolated node in network and increases to 1.87-fold.


## -----------------------------------------------------------------------------
TCGA_E9_A1N5_mirnagene %>%    
  group_by(Target) %>%        
  mutate(gene_expression= max(gene_expression)) %>%
  distinct() %>%
  ungroup() -> TCGA_E9_A1N5_mirnagene

TCGA_E9_A1N5_mirnagene%>%
  filter(gene_expression > 10)->TCGA_E9_A1N5_mirnagene


## ----warning=FALSE, eval=FALSE------------------------------------------------
# TCGA_E9_A1N5_mirnagene %>%
#   priming_graph(competing_count = gene_expression,
#                 miRNA_count = total_read)%>%
#   calc_perturbation(node_name= "ACTB", cycle=10, how= 1.87,limit = 0.1)

## ----warning=FALSE, eval=FALSE------------------------------------------------
# TCGA_E9_A1N5_mirnagene %>%
#   priming_graph(competing_count = gene_expression,
#                 miRNA_count = total_read)%>%
#   calc_perturbation(node_name= "ACTB", cycle=10, how= 30,limit = 0.1)

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----pressure, echo=FALSE, fig.align='center', fig.width=6, fig.height=5, dpi=120----

knitr::include_graphics("gifs/model_vignette.png")

## ----setup, message= FALSE, warning=FALSE-------------------------------------
library(ceRNAnetsim)

## ----Installation, eval = FALSE-----------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("ceRNAnetsim")

## ----echo=TRUE----------------------------------------------------------------
data("minsamp")
minsamp %>% 
  select(1:4)

## ----message= FALSE, warning=FALSE, echo=TRUE---------------------------------
data("minsamp")
minsamp %>% 
  select(competing, Competing_expression) %>% 
  distinct() -> gene_expression

## -----------------------------------------------------------------------------
gene_expression

## ----message= FALSE, warning=FALSE, echo=TRUE---------------------------------
minsamp %>% 
  select(miRNA, miRNA_expression) %>% 
  distinct() -> mirna_expression

## -----------------------------------------------------------------------------
mirna_expression

## ----message= FALSE, warning=FALSE, echo=TRUE---------------------------------
minsamp %>% 
  select(competing, miRNA) -> interaction_simple

## -----------------------------------------------------------------------------
interaction_simple

## -----------------------------------------------------------------------------
interaction_simple %>%
  inner_join(gene_expression, by = "competing") %>%
  inner_join(mirna_expression, "miRNA") -> basic_data

basic_data

## -----------------------------------------------------------------------------
#Convertion of dataset to graph.

priming_graph(basic_data, competing_count = Competing_expression, miRNA_count =miRNA_expression)

## -----------------------------------------------------------------------------

priming_graph(basic_data, competing_count = Competing_expression, 
              miRNA_count =miRNA_expression) %>%
  update_how(node_name = "Gene2", how=2)


## -----------------------------------------------------------------------------

priming_graph(basic_data, competing_count = Competing_expression, 
              miRNA_count =miRNA_expression) %>%
  update_how(node_name = "Gene2", how=2) %>%
  simulate(cycle = 5)


## -----------------------------------------------------------------------------
priming_graph(basic_data, competing_count = Competing_expression, 
              miRNA_count =miRNA_expression) %>%
  update_how(node_name = "Gene2", how=2) %>%
  simulate(cycle = 5)%>%
  as_tibble()%>%
  select(name, initial_count, count_current)

## -----------------------------------------------------------------------------

priming_graph(basic_data, competing_count = Competing_expression, 
              miRNA_count =miRNA_expression) %>%
  update_how(node_name = "Gene2", how=0) %>%
  simulate(cycle = 5)


## -----------------------------------------------------------------------------
minsamp

## -----------------------------------------------------------------------------
priming_graph(minsamp, 
              competing_count = Competing_expression, 
              miRNA_count = miRNA_expression,
              aff_factor = c(energy, seed_type), 
              deg_factor = region)

## ----results='hold'-----------------------------------------------------------
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   update_how(node_name = "Gene4", how = 2) %>% 
   activate(edges)%>%
   # following line is just for focusing on necessary
   # columns to see the change in edge data
   select(3:4,comp_count_pre,comp_count_current)

## -----------------------------------------------------------------------------
data(new_counts) 
new_counts

## -----------------------------------------------------------------------------
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   update_variables(current_counts = new_counts)


## -----------------------------------------------------------------------------
minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(energy, seed_type), 
                deg_factor = region) %>%
  update_how("Gene4", how = 2) 

# OR
# minsamp %>%
#   priming_graph(competing_count = Competing_expression, 
#                 miRNA_count = miRNA_expression,
#                 aff_factor = c(energy, seed_type), 
#                 deg_factor = region) %>%
#   update_variables(current_counts = new_counts) 

## ----echo=FALSE, fig.align='center', fig.width=10, fig.height=8, dpi=120------
knitr::include_graphics("gifs/small_sample_chunk10.png")

## -----------------------------------------------------------------------------
minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(energy, seed_type), 
                deg_factor = region) %>%
  update_how("Gene4", how = 2) %>%
  simulate(cycle=10) #threshold with default 0.

## -----------------------------------------------------------------------------
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   update_how("Gene4", how = 2) %>%
   simulate(cycle=10) %>%
   activate(edges) %>%       #from tidygraph package 
   select(comp_count_list, mirna_count_list) %>%
   as_tibble()  

## ----echo=FALSE, warning=FALSE, message=FALSE---------------------------------
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   update_how("Gene4", how = 2) %>%
   simulate(cycle=10) %>%
   activate(edges) %>%       #from tidygraph package 
   select(comp_count_list, mirna_count_list) %>%
   as_tibble() %>% 
   filter(from==4, to==7) %>% 
   pull(comp_count_list) %>% 
   .[[1]] %>% 
   round()


## -----------------------------------------------------------------------------
minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(energy, seed_type), 
                deg_factor = region) %>%
  update_how("Gene4", how = 2) %>%
  simulate(cycle=3, threshold = 1)

## ----fig.height=4, fig.width=5, warning=FALSE, dpi= 120, fig.align='center'----
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   vis_graph(title = "Minsamp initial Graph")

## ----fig.height=4, fig.width=5, warning=FALSE, dpi= 120, fig.align='center'----
minsamp %>%
   priming_graph(competing_count = Competing_expression, 
                 miRNA_count = miRNA_expression,
                 aff_factor = c(energy, seed_type), 
                 deg_factor = region) %>%
   update_variables(current_counts = new_counts) %>%
   simulate(3) %>%
   vis_graph(title = "Minsamp Graph After 3 Iteration")

## ----fig.height=4, fig.width=5, warning=FALSE, message=FALSE, eval=FALSE------
# minsamp %>%
#    priming_graph(competing_count = Competing_expression,
#                  miRNA_count = miRNA_expression,
#                  aff_factor = c(energy, seed_type),
#                  deg_factor = region) %>%
#    update_variables(current_counts = new_counts) %>%
#    simulate_vis(3, title = "Minsamp Graph After Each Iteration")

## ----sessioninfo--------------------------------------------------------------
sessionInfo()

utils::globalVariables(c("name", "count_current", "initial_count", ".",
                         "from", "comp_count_list", "dif", "non_zero",
                         "effect", "iter", "nodes", "eff_count",
                         "effect_current", "comp_count_current",
                         "mirna_count_current", "Competing_name",
                         "comp_count_pre", "miRNA_name", "dummy",
                         "competing", "miRNA", "nodes", "to",
                         "afff_factor", "mirna_count_per_dep",
                         "degg_factor", "mirna_count_per_comp",
                         "effect_list", "mirna_count_list",
                         "count_pre", "type", "node_id",
                         "changes_variable", "x", "y",
                         "mirna_count_pre", "effect_pre", "len",
                         "change", "node", "top_nodes", "initial_exp", "final_exp"))

#' Provides visualisation of the graph in addition to simulate function.
#'
#' simulate_vis provides visualisation of the graph in addition to simulate function.
#'
#' @importFrom purrr pmap
#' @importFrom ggplot2 ggsave
#' @details simulate_vis gives the last graph object and each iterations' image.
#'
#' @return It gives a graph and the images of states in each iteration until the end of the simulation.
#'
#' @param input_graph The graph object that processed in previous steps.
#' @param cycle Optimal iteration number for gaining steady-state.
#' @param Competing_color The color of competing elements on the graph with "green" default.
#' @param mirna_color The color of miRNAs on the graph with "orange" default.
#' @param Upregulation The color of Upregulated elements on the graph with "red" default.
#' @param Downregulation The color of Downregulated elements on the graph with "blue" default.
#' @param title Title of the given graph.
#' @param layout The layout that will be used for visualisation of the graph.
#' @param threshold absolute minimum amount of change required to be considered as up/down regulated element
#' @param save provides to save graph output 
#'
#' @examples
#'
#' # When does the system gain steady-state conditions again?
#'
#' ## new_counts, the dataset that includes the current counts of nodes.
#'
#' data("minsamp")
#' data("new_counts")
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression)%>%
#'   update_variables(new_counts)%>%
#'   simulate_vis()
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression, 
#'     aff_factor = c(seed_type,energy), deg_factor = c(region))%>%
#'   update_variables(new_counts)%>%
#'   simulate_vis(cycle = 12)
#'
#' @export


simulate_vis <- function(input_graph, cycle=1, threshold = 0, save = FALSE, Competing_color = "green", mirna_color = "orange", Upregulation = "red", Downregulation = "blue", title = "GRAPH", layout= "kk"){



  for(i in seq_len(cycle)){


    input_graph%>%
      tidygraph::activate(edges)%>%
      tidygraph::group_by(to)%>%
      tidygraph::mutate(mirna_count_per_comp = mirna_count_current*comp_count_current*afff_factor/sum(comp_count_current*afff_factor), mirna_count_per_comp = ifelse(is.na(mirna_count_per_comp), 0, mirna_count_per_comp))%>%
      tidygraph::ungroup()%>%
      tidygraph::mutate(effect_pre = effect_current, effect_current = mirna_count_per_comp*degg_factor)%>%
      tidygraph::group_by(from)%>%
      tidygraph::mutate(comp_count_pre = ifelse(comp_count_current< 0, 1, comp_count_current), comp_count_current = comp_count_pre - (sum(effect_current)-sum(effect_pre)), comp_count_current = ifelse(comp_count_current< 0, 1, comp_count_current))%>%
      tidygraph::ungroup()%>%
      tidygraph::mutate(comp_count_list = pmap(list(comp_count_list, comp_count_current), c), effect_list = pmap(list(effect_list, effect_current), c), mirna_count_list = pmap(list(mirna_count_list, mirna_count_current), c))%>%
      tidygraph::activate(nodes)%>%
      update_nodes(limit = threshold)-> input_graph

    vis_graph(input_graph, Competing_color, mirna_color, Upregulation, Downregulation, title = paste(title, "-",i), layout)-> graph_vis

    print(graph_vis)
    
    if(save){
      
      ggsave(paste(title, ".png", sep = ""))
    }
  }
  input_graph

}

#' Finds potential affecting node for given particular target.
#'
#' @importFrom furrr future_map future_map_dfr
#' @importFrom future plan
#' @importFrom rlang set_names
#' @importFrom igraph gsize degree V 'V<-'
#' @importFrom purrr map_dbl
#' @importFrom tidyr unnest
#'
#' @return It gives a tibble form dataset that includes parturbation node (source) and
#' change in count of targeting node 
#'
#' @details Lists potential targeting nodes by running 
#' find_affected_nodes function for all nodes in network.
#'
#' @param input_graph The graph object that was processed with priming_graph function.
#' @param how The change of count (expression) of the given node in terms of fold change.
#' @param cycle The iteration of simulation.
#' @param limit The minimum fold change which can be taken into account for
#' perturbation calculation on all nodes in terms of percentage.
#' @param fast specifies percentage of affected target in target expression.
#' For example, if fast = 1, the nodes that are affected from miRNA repression
#' activity more than one percent of their expression is determined as subgraph.
#' @param top Determines how many nodes most affected will be evaluated.
#' @param target The target node in which is being investigated.
#'
#' @examples
#'
#' data('midsamp')
#' 
#' midsamp%>%
#' priming_graph(competing_count = Gene_expression, 
#'               miRNA_count = miRNA_expression)%>%
#' find_targeting_nodes(how = 2, 
#'                      cycle = 2, 
#'                      target = "Gene1", 
#'                      top = 2)
#'
#'
#' @export


find_targeting_nodes <- function(input_graph, how = 2, cycle = 1, limit = 0, fast = 0, top = 5, target = NULL) {
  

  empty_result <- list(dplyr::tibble(top_nodes = NA, change = NA, initial_exp = NA, final_exp=NA))
  
  if (fast == 0) {
    
    result <- input_graph %>% 
      activate(nodes) %>% 
      tidygraph::mutate(eff_count = future_map(V(input_graph)$name,~find_affected_nodes(input_graph, .x, how, cycle, limit, top))) %>%
      tibble::as_tibble() %>% 
      unnest(eff_count)
  }
  
  if (fast != 0) {
    
    result <- input_graph %>% 
      tidygraph::activate(edges) %>% 
      tidygraph::mutate(fast = ifelse(100 * effect_current/comp_count_current > fast, TRUE, FALSE)) %>%
      tidygraph::morph(to_subgraph, fast) %>%
      { if( igraph::gsize(crystallise(.)$graph[[1]])==0) stop("Your argument fast removed all nodes. Please try lower fast argument")   else .  }%>%
      tidygraph::activate(nodes) %>%
      tidygraph::mutate(degree = tidygraph::centrality_degree(mode = "all")) %>%
      tidygraph::filter(degree > 0) %>% 
      tidygraph::mutate(eff_count = tidygraph::map_bfs(node,.f = function(graph, node, ...) {
        find_affected_nodes(graph, V(graph)$name[node], how, cycle,limit, top)})) %>% 
      tidygraph::unmorph() %>% 
      tidygraph::mutate(len = map_dbl(eff_count, length), eff_count = ifelse(len == 0, empty_result, eff_count)) %>%
      tidygraph::as_tibble() %>% 
      unnest() %>% 
      dplyr::select(-degree, -len)
    
  }
  
  if (is.null(target)){
    
    result <- result%>%
      select(source, top_nodes, change, initial_exp, final_exp)%>%
      unnest(cols = c(top_nodes, change, initial_exp, final_exp))%>%
      rename("targeting_node" = "source")
  }
  
  else{
    
    result <- result%>%
      select(source, top_nodes, change, initial_exp, final_exp)%>%
      unnest(cols = c(top_nodes, change, initial_exp, final_exp))%>%
      filter(top_nodes == target)%>%
      rename("targeting_node" = "source")
    
    
    
  }
  
  return(result)
}

#' Converts the count value of the given node.
#'
#' this function converts the count value of the given node.
#'
#' @importFrom igraph E 'E<-'
#' @return the graph object.
#'
#' @details update_how function calculates the current value of
#' given mirna or gene node on the graph object. User must specify
#' current value as fold change.
#'
#' @param input_graph The graph object that processed in previous step/s.
#' @param node_name The name of the node whose count is to be changed.
#' @param how The change in terms of fold change.
#' @param knockdown specifies gene knockdown with default TRUE
#'
#' @examples
#'
#' data('minsamp')
#'
#' priming_graph(minsamp, Competing_expression, miRNA_expression)%>%
#'    update_how('Gene1',3)
#'
#'  priming_graph(minsamp, Competing_expression, miRNA_expression,
#'       aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'    update_how('Gene1', 3)
#'
#'    priming_graph(minsamp, Competing_expression, miRNA_expression,
#'        aff_factor = c(seed_type,energy), deg_factor = region)%>%
#'   update_how('Gene1', how=0, knockdown= TRUE)
#'
#'
#'
#' @export

update_how <- function(input_graph, node_name, how, knockdown = TRUE) {

  if (how < 0)
    stop("Fold change should not be less than zero.
                     Please use decimal values for decrease. e.g. 0.5 for 2-fold decrease",
         call. = FALSE)

  if (how == 0 & knockdown) {

    return(input_graph %>% gene_knockdown(node_name) %>% update_nodes())
  }

  if (node_name %in% E(input_graph)$Competing_name) {

    input_graph <- input_graph %>% tidygraph::activate(edges) %>% tidygraph::mutate(comp_count_current = ifelse(node_name ==
                                                                                                                  E(input_graph)$Competing_name, comp_count_current * how, comp_count_current))

  } else if (node_name %in% E(input_graph)$miRNA_name) {

    input_graph <- input_graph %>% tidygraph::activate(edges) %>% tidygraph::mutate(mirna_count_current = ifelse(node_name ==
                                                                                                                   E(input_graph)$miRNA_name, mirna_count_current * how, mirna_count_current))

  } else {

    stop(paste("Given node name", node_name, "was not found! Please check it.",
               sep = " "))

  }

  return(input_graph %>% update_nodes())
}



#' Knocks down given node.
#'
#' @importFrom igraph E
#' @return the graph object.
#'
#' @details knocks down a given gene target.
#'
#' @param input_graph The graph object that processed in previous step/s.
#' @param node_name The name of the node whose count is to be knocked down.
#'
#' @keywords internal
#'

gene_knockdown <- function(input_graph, node_name) {

  input_graph <- input_graph %>% tidygraph::activate(edges) %>% tidygraph::mutate(comp_count_current = ifelse(node_name ==
                                                                                                                E(input_graph)$Competing_name, 0, comp_count_current))

  return(input_graph %>% update_nodes())

}

---
title: "The auxiliary commands which can help to the users"
author: "Selcen Ari"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
   toc: true
vignette: >
  %\VignetteIndexEntry{auxiliary_commands}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, warning= FALSE, message=FALSE}
library(ceRNAnetsim)
```

# 1. Introduction

In the other package vignettes, usage of ceRNAnetsim is explained in details. But in this vignette, some of commands which facitate to use of other vignettes.

# 2. Installation

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("ceRNAnetsim")
```

# 3. Selection of perturbing element from dataset

```{r}
data("TCGA_E9_A1N5_tumor")
data("TCGA_E9_A1N5_normal")
data("mirtarbasegene")
data("TCGA_E9_A1N5_mirnanormal")
```

## 3.1. Selection of HIST1H3H gene at vignette [How does the system behave in mirtarbase dataset without interaction factors?](https://selcenari.github.io/ceRNAnetsim/articles/mirtarbase_example.html)

```{r}
TCGA_E9_A1N5_mirnanormal %>%
  inner_join(mirtarbasegene, by= "miRNA") %>%
  inner_join(TCGA_E9_A1N5_normal, 
             by = c("Target"= "external_gene_name")) %>%
  select(Target, miRNA, total_read, gene_expression) %>%
  distinct() -> TCGA_E9_A1N5_mirnagene
```


```{r}
TCGA_E9_A1N5_tumor%>%
  inner_join(TCGA_E9_A1N5_normal, by= "external_gene_name")%>%
  select(patient = patient.x, 
         external_gene_name, 
         tumor_exp = gene_expression.x, 
         normal_exp = gene_expression.y)%>%
  distinct()%>%
  inner_join(TCGA_E9_A1N5_mirnagene, by = c("external_gene_name"= "Target"))%>%
  filter(tumor_exp != 0, normal_exp != 0)%>%
  mutate(FC= tumor_exp/normal_exp)%>%
  filter(external_gene_name== "HIST1H3H")

#HIST1H3H: interacts with various miRNA in dataset, so we can say that HIST1H3H is non-isolated competing element and increases to 30-fold.

```

## 3.2. Selection of ACTB gene at vignette [How does the system behave in mirtarbase dataset without interaction factors?](https://selcenari.github.io/ceRNAnetsim/articles/mirtarbase_example.html)

```{r}
TCGA_E9_A1N5_tumor%>%
  inner_join(TCGA_E9_A1N5_normal, by= "external_gene_name") %>%
  select(patient = patient.x, 
         external_gene_name, 
         tumor_exp = gene_expression.x, 
         normal_exp = gene_expression.y) %>%
  distinct() %>%
  inner_join(TCGA_E9_A1N5_mirnagene, 
             by = c("external_gene_name"= "Target")) %>%
  filter(tumor_exp != 0, normal_exp != 0) %>%
  mutate(FC= tumor_exp/normal_exp) %>%
  filter(external_gene_name == "ACTB")

#ACTB: interacts with various miRNA in dataset, so ACTB is not isolated node in network and increases to 1.87-fold.

```

## 4. Determination of ACTB gene perturbation efficiency with different expression level changes

Firstly, clean dataset as individual gene has one expression value. And then filter genes which have expression values greater than 10.

```{r}
TCGA_E9_A1N5_mirnagene %>%    
  group_by(Target) %>%        
  mutate(gene_expression= max(gene_expression)) %>%
  distinct() %>%
  ungroup() -> TCGA_E9_A1N5_mirnagene

TCGA_E9_A1N5_mirnagene%>%
  filter(gene_expression > 10)->TCGA_E9_A1N5_mirnagene

```

We can determine perturbation efficiency of an element on entire network as following:

```{r, warning=FALSE, eval=FALSE}
TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read)%>%
  calc_perturbation(node_name= "ACTB", cycle=10, how= 1.87,limit = 0.1)
```

On the other hand, the perturbation eficiency of ATCB gene is higher, when this gene is regulated with 30-fold upregulation like in HIST1H3H.

```{r, warning=FALSE, eval=FALSE}
TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read)%>%
  calc_perturbation(node_name= "ACTB", cycle=10, how= 30,limit = 0.1)
```

# 5. Session Info
```{r sessioninfo}
sessionInfo()
```

---
title: "A TCGA dataset application"
author: "Selcen Ari"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
   toc: true
vignette: >
  %\VignetteIndexEntry{An TCGA dataset application}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: ref.bib
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


# 1. Introduction

This vignette is about the integration of gene and miRNA pairs and their expression dataset and analysis. The sample dataset in this demonstration, which contains human miRNA:target pairs, was retrieved from [miRTarBase](http://mirtarbase.mbc.nctu.edu.tw/php/download.php) website (Release 7.0).

```{r message= FALSE, warning=FALSE}
library(ceRNAnetsim)
```

# 2. Installation

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("ceRNAnetsim")
```

# 3. Integration of dataset which includes only miRNA and gene expression values

## 3.1. miRNA:target pairs

```{r, warning= FALSE, message=FALSE}
data("mirtarbasegene")
head(mirtarbasegene)
```

**NOTE** if the mirna:target dataset includes miRNA genes as targets, the `priming_graph()` function can fail. Because, the function define to miRNAs and targets without distinguishing between uppercase or lowercase. 

## 3.2. Gene expression in normal and tumor samples

The gene and mirna expression counts of patient barcoded with TCGA-E9-A1N5 is retrieved from TCGA via `TCGAbiolinks` package  [@colaprico2015tcgabiolinks] from `Bioconductor`. The instructions of retrieving data can be found at `TCGAbiolinks` manual. 

For this step you don't have to use TCGA data, any other source or package can be utilized.

```{r}
data("TCGA_E9_A1N5_normal")
head(TCGA_E9_A1N5_normal)
```

```{r}
data("TCGA_E9_A1N5_tumor")
head(TCGA_E9_A1N5_tumor)
```

## 3.3. miRNA expression data

```{r}
data("TCGA_E9_A1N5_mirnatumor")
head(TCGA_E9_A1N5_mirnatumor)
```

```{r}
data("TCGA_E9_A1N5_mirnanormal")
head(TCGA_E9_A1N5_mirnanormal)
```

Here's the summary of size of each dataset

|Dataset name| Number of rows |
|------------|----------------|
| `mirtarbasegene`           | `r nrow(mirtarbasegene)`           |
| `TCGA_E9_A1N5_normal`      | `r nrow(TCGA_E9_A1N5_normal)`      |
| `TCGA_E9_A1N5_tumor`       | `r nrow(TCGA_E9_A1N5_tumor)`       |
| `TCGA_E9_A1N5_mirnanormal` | `r nrow(TCGA_E9_A1N5_mirnanormal)` |
| `TCGA_E9_A1N5_mirnatumor`  | `r nrow(TCGA_E9_A1N5_mirnatumor)`  |

## 3.4. Integrating and analysing data

All of these datasets are integrated using the code below resulting in  miRNA:target dataset that contains miRNA and gene expression values.

```{r}
TCGA_E9_A1N5_mirnagene <- TCGA_E9_A1N5_mirnanormal %>%
  inner_join(mirtarbasegene, by= "miRNA") %>%
  inner_join(TCGA_E9_A1N5_normal, 
             by = c("Target"= "external_gene_name")) %>%
  select(Target, miRNA, total_read, gene_expression) %>%
  distinct() 
```

Note: Some of genes have expression values more than one because some of tissue samples were sequenced in two medium separately. So, we select maximum expression values of that genes at following:

```{r, echo=FALSE}

TCGA_E9_A1N5_mirnagene%>%
  group_by(Target, miRNA)%>%
  count()%>%
  filter(n==2)

TCGA_E9_A1N5_mirnagene %>%    
  group_by(Target) %>%        
  mutate(gene_expression= max(gene_expression)) %>%
  distinct() %>%
  ungroup() -> TCGA_E9_A1N5_mirnagene

```

```{r}
head(TCGA_E9_A1N5_mirnagene)
```

When we compared the two gene expression dataset of TCGA-E9A1N5 patient, and selected a gene which has 30-fold increased expression, (gene name: HIST1H3H), this gene node will be used in the example. Note that the selected node must not be isolated one. If the an isolated node is selected the change in expression will not propagate in network. (You can see commands for node selection in the vignette [The auxiliary commands which can help to the users](https://selcenari.github.io/ceRNAnetsim/articles/auxiliary_commands.html))

Optionally, you can filter the low expressed gene nodes because they are not effective elements.

```{r}
TCGA_E9_A1N5_mirnagene <- TCGA_E9_A1N5_mirnagene%>%
  filter(gene_expression > 10)
```

The analysis is performed based on amounts of miRNAs and targets as seen. Firstly, we tried to find optimal iteration for the network when simulation start with *HIST1H3H* node. As an example, `simulation()` function was used with `cycle = 5` argument, this argument can be arranged according to network. Note that it can be appropriate that using greater number of `cycle` for comprehensive network objects.

```{r, warning=FALSE, fig.height=5, fig.width=6, fig.align='center', warning=FALSE}

simulation_res_HIST <- TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read) %>% 
  update_how(node_name = "HIST1H3H", how =30) %>% 
  simulate(5)

simulation_res_HIST%>%
  find_iteration(plot=TRUE)

```

The graph was shown that the change in expression level of HIST1H3H results in weak perturbation efficiency, despite 30-fold change. The code shown below can be used for calculation of fold changes after simulation HIST1H3H gene to 30 fold:

```{r}

simulation_res_HIST%>%
  as_tibble()%>%
  mutate(FC= count_current/initial_count)%>%
  arrange(desc(FC))
```

And then, we tried to simulate the network with the gene which has higher expression value. For this, we selected *ACTB* node as shown in [The auxiliary commands which can help to the users](https://selcenari.github.io/ceRNAnetsim/articles/auxiliary_commands.html)

```{r, warning=FALSE, fig.height=5, fig.width=6, fig.align='center', warning=FALSE}
simulation_res_ACTB <- TCGA_E9_A1N5_mirnagene %>% 
  priming_graph(competing_count = gene_expression, 
                miRNA_count = total_read) %>% 
  update_how(node_name = "ACTB", how =1.87) %>% 
  simulate(5)

simulation_res_ACTB%>%
  find_iteration(plot=TRUE)
```

Following codes are shown entire gene fold changes after simulation ACTB gene to 1.87 fold:

```{r}

simulation_res_ACTB%>%
  as_tibble()%>%
  mutate(FC= count_current/initial_count)%>%
  arrange(desc(FC))
```

Note: it can be useful that you look at [The auxiliary commands which can help to the users](https://selcenari.github.io/ceRNAnetsim/articles/auxiliary_commands.html) for perturbation efficiency of ACTB gene by simulation with same conditions and different expression changes.

## 3.5. The sum of two conditions:

In a real biological sample, we tested perturbation efficiencies of two genes;
* one with low expression but high fold change (HIST1H3H, 30-fold increase in tumor) 
* another one with high expression but small change in expression level (ACTB, 1.87-fold increase in tumor)

With these two samples, it has been obtained that expression values of genes, rest of the perturbed gene, changed slightly.

Despite high fold change, former gene caused little perturbation. When the perturbation efficiencies of both of these genes are analysed, it has been oberved that HIST1H3H does not affect the other genes in given limit. On the contrary, high expressing gene with very low fold increase in tumor causes greater perturbation in the network. Additionaly, the perturbation efficiency of ACTB gene is quite high from HIST1H3H with 30-fold change, when ACTB is simulated with 30 fold-change.

Thus, if the perturbed node has lower target:total target ratio in group or groups, the efficiency of it can be weak, or vice versa. The efficiency of ACTB gene may be high for this reason, in comparison with HIST1H3H perturbation. In fact, it has been observed that ACTB has not strong perturbation efficiency too. This could be arisen from low miRNA:target ratio or ineffective target nodes which have very low expression levels.


# 4. Dataset (`huge_example`) which includes miRNA and gene expressions and miRNA:target interaction factors

## 4.1. Description of the *huge_example* dataset

Interactions between miRNAs and their targets can be analyzed after the integration of miRNA and targets via various datasets. As an example, we prepared the *huge_example* dataset. It was generated by integrating:

* Next-generation RNA sequencing data of a breast cancer patient from TCGA (patient id:TCGA_A7_A0CE) 
* The  microRNA expression values of the same breast cancer patient.
* High-throughput miRNA:target determination datasets. We utilised the microRNA:target gene dataset which was integrated from high-throughput experimental studies([CLASH](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3650559/) & [CLEAR-CLiP](https://www.nature.com/articles/ncomms9864) methods). These datasets give exact microRNA:target pairs because of these methods base on chimeric reading of pairs after binding of microRNAs and their targets. For this reason, the datasets contain detailed information for microRNA:target pairs such as interacted bases (in terms of degree of complementarity) on microRNAs and target mRNAs, location of interaction region (seed) on mRNA and estimated binding energies of pairs.


Below, only 6 rows from total of 26,176 rows are shown.

```{r, message=FALSE, warning=FALSE}
data("huge_example")
head(huge_example)
```

## 4.2. Select a node as trigger

The node that initiates simulation can be determined according your interest or research.

The dataset, which is a data frame, can be manipulated with [tidyverse packages](https://www.tidyverse.org/packages/). As an example, competing RNAs targeted by less than 5 miRNAs are eliminated to make the network manageable size.

```{r}
filtered_example <- huge_example %>%
  add_count(competing) %>%
  filter(n > 5) %>%
  select(-n)

head(filtered_example)
```

On the other hand, we chose the node *GAPDH* according to interaction count of the nodes. With the simulation, the graph was visualized after node *GAPDH* was increased to five fold.

```{r, fig.height=5, fig.width=6, fig.align='center', warning=FALSE}
simulation_GAPDH <- filtered_example %>%
  priming_graph(competing_count = competing_counts, 
                miRNA_count = mirnaexpression_normal, 
                aff_factor = Energy) %>%
  update_how("GAPDH", 5) 

simulation_GAPDH%>%
  vis_graph(title = "Distribution of GAPDH gene node")
```

Let's visualize each step of simulation via `simulate_vis()` function.

```{r, fig.height=5, fig.width=6, fig.align='center', warning=FALSE, fig.show='hide'}

simulation_GAPDH%>%
  simulate_vis(title = "GAPDH over expression in the real dataset", 3)
```

![GAPDH over expression in real dataset](gifs/realexample.gif#center)

Now, we can track changes in expression levels at every node for 3 cycles when GAPDH is overexpressed 5-fold.

- After increase in GAPDH expression level in the first graph, the responses of the other competing elements to the GAPDH distributions were calculated. 

- The changing regulations (up or down) were observed as a result of interactions in the second graph.

- When three graphs were carefully compared to each other, it can be observed that the expression levels of nodes change continuously at each stage.

# 5. Finding perturbation efficiency on an experimental dataset

`find_node_perturbation()` runs `calc_perturb` on all nodes in the network in parallel with help of the `future` and `furrr` packages. In this vignette, the function is demonstrated on the `midsamp` data. This dataset is not comparable to actual biological miRNA:target gene datasets in size and complexity. Although `find_node_perturbation()` runs in parallel it might take long time to run in real huge biological datasets. 

In real biological datasets, more complex interactions whether functional or non-functional could be observed. We have improved our approach with `fast` argument in `find_node_perturbation()` based on selection of elements that could be affected from perturbation. In this fucntion, `fast` argument specifies the percentage of the competing amount that can be affected within the initial competing amount and acts as a selection parameter. For instance, in filtered example data:

```{r, warning=FALSE}

entire_perturbation <- filtered_example%>%
  priming_graph(competing_count = competing_counts, miRNA_count = mirnaexpression_normal)%>%
  find_node_perturbation(how=5, cycle=3, fast = 15)%>%
  select(name, perturbation_efficiency, perturbed_count)
  
```

```{r, warning=FALSE}

entire_perturbation%>%
  filter(!is.na(perturbation_efficiency), !is.na(perturbed_count))%>%
  select(name, perturbation_efficiency, perturbed_count)
```


# 6. Session Info
```{r sessioninfo}
sessionInfo()
```

## References

---
title: "Calculating Number of Iterations Required to Reach Steady-State"
author: "Selcen Ari"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
   toc: true
vignette: >
  %\VignetteIndexEntry{A Suggestion: How to Find the Appropriate Iteration for Simulation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# 1. Introduction

In the `ceRNAnetsim` package, regulations of miRNA:target pairs are observed via direct or indirect interactions of elements in network. In this approach, change in expression level of single gene or miRNA can affect the whole network via "ripple effect". So, when the change is applied the system, it affects to primary neighborhood firstly, and then propagates to further neighborhoods.

In the simple interaction network like *minsamp*, the ripple effect could be observed when expression level of *Gene4* changes and subsequently effecting other genes. In the non-complex networks like *minsamp*, the steady-state condition can be provided easily, after network disturbed.

In this vignette, first, we demonstrate a suggestion to determine simulation iteration of existing dataset for gaining steady state after perturbing the network. Additionally, new approach which is useful for defining significant of nodes in terms of perturbation of network is elucidated.

```{r message= FALSE, warning=FALSE}
library(ceRNAnetsim)
library(png)
```

# 2. Installation

```{r, message=FALSE, warning= FALSE, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("ceRNAnetsim")
```

# 3. Comparison of gaining steady-state durations of `middle` and `minimal` datasets

```{r, fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'}
data("minsamp")

minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression) %>%
  update_how("Gene4",2) %>%
  simulate_vis(title = "Minsamp: Common element as trigger", cycle = 15)


minsamp %>%
  priming_graph(competing_count = Competing_expression, 
                miRNA_count = miRNA_expression) %>%
  update_how("Gene4",2) %>%
  simulate(cycle = 5)
```

![Minsamp common target perturbation](gifs/con_iter_chunk3.gif#center)

For example, in *minsamp* dataset, the steady-state is occurred at iteration-14 (as seen above: after iteration-13, it is observed that there are only orange (miRNAs) and green (competing genes) nodes in network. In this case, genes have new regulated (steady) expression values while expression values of microRNAs are same in comparison with initial case.).

However, when network is larger and interactions are more complex, the number of iterations required to reach steady-state may increase. While at cycle 14 *minsamp* dataset has reached steady-state, the *midsamp* (middle sized sample) dataset has not reached steady-state after 15 cycles. In the example below, in *midsamp* data, *Gene17* is upregulated 2 fold as a trigger and simulation is run for 15 cycles.

```{r}
data("midsamp")

midsamp
```

```{r, fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'}
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17",2) %>%
  simulate_vis(title = "Midsamp: Gene with higher degree as trigger", 15)
```

![Midsamp Gene17 perturbation followed with 15 iterations](gifs/con_iter_chunk4.gif)

## 3.1. Suggestion for simulation iteration

Guessing or performing trial and error for large networks is not practical, thus we developed a function which calculates optimal iteration in a network after trigger and simulation steps. `find_iteration()` function analyses the simulated graph and suggests the iteration at which maximum number of nodes are affected. An important argument is `limit` which sets the threshold below which is considered "no change", in other words, any node should have level of change greater than the threshold to be considered "changed". Please be aware that small threshold values will cause excessively long calculation time especially in large networks.

In the example below, *Gene2* is upregulated 2-fold and then iteration number at which maximum number of nodes affected will be calculated. The search for iteration number will go up to 50. Also, since we are searching for maximal propagation, limit is set to zero.

```{r}
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene2",2) %>%
  simulate(50) %>% 
  find_iteration(limit=0)

```

NOTE: You can edit the dataset manually. You can change Gene2 expression value as 20000 and save that as a new dataset (midsamp_new_counts).

You can use the dataset that includes new expression values of miRNAs and genes.

```{r}
data("midsamp_new_counts")

midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_variables(current_counts = midsamp_new_counts) %>%
  simulate(50) %>% 
  find_iteration(limit=0)
```

`find_iteration()` function will return a single number: the iteration number at which maximum propagation is achieved. If `plot=TRUE` argument is used then the function will return a plot which calculates percent of perturbed nodes for each iteration number. The latter can be used for picking appropriate number of cycles for `simulate()` function.

```{r, fig.align='center', fig.width=5, fig.height=3, dpi= 120}
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17",2) %>%
  simulate(50) %>% 
  find_iteration(limit=0)

```

## 3.2. Find appropriate iteration number with `find_iteration` and then simulate accordingly

As shown in plot above, if "Gene17" is upregulated 2-fold, the network will need around 22 iterations to reach the steady-state. Since we have an idea about appropriate iteration number, let's use `simulate()` function and iterate for 25 cycles using same trigger (Gene17 2-fold):

```{r, fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'}
midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17", 2) %>%
  simulate_vis(title = "Midsamp: Gene17 2 fold increase, 25 cycles", 25)
```

![Midsamp Gene17 perturbation with 25 iteration](gifs/con_iter_chunk7.gif#center)

Note: If you ignore decimal change in gene expression, `threshold` argument can be used. With this method, system reaches steady-state early.

```{r, fig.align='center', fig.height=5, fig.width=6, warning=FALSE, message=FALSE, fig.show='hide'}

midsamp %>%
  priming_graph(Gene_expression, miRNA_expression) %>%
  update_how("Gene17", 2) %>%
  simulate_vis(title = "Midsamp: Gene17 2 fold increase, 6 cycles", 6, threshold = 1)

```

![Midsamp Gene17 perturbation with 6 iteration with threshold](gifs/con_iter_chunk8.gif#center)

The workflow that is aforementioned in this vignette should be considered as suggestion. Because the `cycle` is a critical argument that is used with `simulate()` function and affects results of analysis. In light of this vignette and functions, the approach can be developed according to dataset.


# 4. What is perturbation efficiency?

The perturbation efficiency means that the disturbance and propagation efficiency of an element in the network. In a given network not all nodes have same or similar perturbation efficiency. Changes in some nodes might propagate to whole network and for some nodes the effect might be limited to small subgraph of the network. Not only topology but also miRNA:target interaction dynamics determine perturbation efficiency.

- Expression level and type of trigger element plays a crucial role. The trigger element can be an miRNA or competing target. The perturbation efficiency is affected from ratio of miRNA amount to sum of expression levels of its targets. Also, amount of competing element among whole competing elements is important since it determines distribution of miRNA. For example, if trigger is an miRNA with expression level of 1500 and if sum of expression levels of its targets is 1000000, then this miRNA will not perturb its neighborhood efficiently. So, miRNA:target ratio is important for regulation of interaction network.

- In a biological system, the miRNA:target interactions does not depend solely on stoichiometry, unfortunately. miRNAs affect the targets via degradation or inhibition after the binding. The experimental studies have shown that the features of miRNA:target interactions determine the binding and degradation efficiency. For example, binding energy between miRNA and target and seed structure of miRNA determine the binding efficiency of complex. In addition, the binding region on the target affects the degradation of target. 

- On the other hand, interaction factors affecting binding and degradation of miRNA to its targets also have impact one efficiency of perturbation of the change. For instance, if a node has very low binding affinity to targeting miRNA, change in expression level of that node will cause weak or no perturbation.

Thus, we developed functions which can calculate perturbation efficiency of a given node or all nodes. `calc_perturbation()` function calculates perturbation efficiency for given trigger (*e.g.* Gene17 2-fold). `find_node_perturbation()` function screens the whole network and calculate perturbation efficiency of all nodes.

## 4.1. How does the `calc_perturbation()` work?

This function works for a given node from network. It calculates and returns two values:

* **perturbation efficiency** : mean of percentage of expression changes of all elements except trigger in the network 
* **perturbed count** : count of disturbed nodes for given iteration number and limit (the minimum change in expression level needed to be considered "disturbed" or "changed"). 

In the example below, "Gene17" is up-regulated 3-fold in *midsamp* dataset where `Energy` and `seeds` columns are used for calculating affinity effect and `targeting_region` columns is used for calculating degradation effect. The network will be iterated over 30 times and number of disturbed nodes (as taking into account nodes that have changed more than the value of the threshold (0.1 percentage in terms of the change)) will be counted.


```{r}
midsamp %>%
  priming_graph(competing_count = Gene_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds), 
                deg_factor = targeting_region) %>%
  calc_perturbation("Gene17", 3, cycle = 30, limit = 0.1)
```

If you are interested in testing various fold change values of a given node, then we can use `map` (actually parallelized version `future_map`) to run function for set of input values.

First, let's keep the primed version of graph in an object

```{r}
primed_mid <- midsamp %>%
  priming_graph(competing_count = Gene_expression, 
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds), 
                deg_factor = targeting_region)
```

Now, let's calculate perturbation efficiency caused by 2-fold to 10-fold increase in Gene17

```{r, eval=FALSE}
# for parallel processing
# future::plan(multiprocess) 

seq(2,10) %>%
  rlang::set_names() %>% 
  furrr::future_map_dfr(~ primed_mid %>% calc_perturbation("Gene17", .x, cycle = 30, limit = 0.1), .id="fold_change")
```

If you're interested in screening nodes instead of fold changes then you don't have to write a complicated `map` command, there's already a function available for that purpose.

## 4.2. A Short-cut: Finding perturbation efficiencies for whole nodes of network

The `find_node_perturbation()` function calculates the perturbation efficiency and perturbed node count of each node in network. 

In the example below, each node is increased 2-fold and tested for perturbation efficiency for 4 cycles with threshold of 0.1

```{r, warning=FALSE}
midsamp %>%
  priming_graph(competing_count = Gene_expression,
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds),
                deg_factor = targeting_region) %>%
  find_node_perturbation(how = 3, cycle = 4, limit = 0.1)%>%
  select(name, perturbation_efficiency, perturbed_count)
```

On the other hand, some of nodes in network might not be affected from perturbation because of low expression or weak interaction factors. In this case, `fast` argument can be used. Argument `fast` calculate affected expression percent of the targets and perturbation calculation is not ran for these elements in network, if that percentage value is smaller than given `fast` value.

```{r}
midsamp %>%
  priming_graph(competing_count = Gene_expression,
                miRNA_count = miRNA_expression,
                aff_factor = c(Energy,seeds),
                deg_factor = targeting_region) %>%
  find_node_perturbation(how = 3, cycle = 4, limit = 0.1, fast=5)%>%
  select(name, perturbation_efficiency, perturbed_count)
```

The results of the `find_node_perturbation()` will list effectiveness or importance of nodes in the network. This function can help selecting nodes which will have effective perturbation in network.

# 5. Session Info
```{r sessioninfo}
sessionInfo()
``` 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{midsamp}
\alias{midsamp}
\title{midsamp}
\format{
A data frame with 7 variables and 26 observation of them:
\describe{
  \item{Genes}{symbol of gene}
  \item{miRNAs}{symol of miRNA}
  \item{Gene_expression}{Expression values of competing gene}
  \item{miRNA_expression}{Expression value of miRNA}
  \item{seeds}{Coefficient for efficiency of seed type of miRNA:target interaction}
  \item{targeting_region}{Coefficient for efficiency of location on target}
  \item{Energy}{Energy of miRNA:target binding}
  }
}
\source{
Dataset was created by us.
}
\description{
middle sized sample dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_how.R
\name{gene_knockdown}
\alias{gene_knockdown}
\title{Knocks down given node.}
\usage{
gene_knockdown(input_graph, node_name)
}
\arguments{
\item{input_graph}{The graph object that processed in previous step/s.}

\item{node_name}{The name of the node whose count is to be knocked down.}
}
\value{
the graph object.
}
\description{
Knocks down given node.
}
\details{
knocks down a given gene target.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{TCGA_E9_A1N5_mirnatumor}
\alias{TCGA_E9_A1N5_mirnatumor}
\title{TCGA_E9_A1N5_mirnatumor}
\format{
Classes tbl_df, tbl and data.frame with 648 observation of 6 variables:
\describe{
  \item{barcode}{Sample, tumor tissue, barcode of patient based on TCGA}
  \item{mirbase_ID}{mirbase id of miRNA}
  \item{miRNA}{miRNA name}
  \item{Precusor}{Precusor id of miRNA which is given in miRNA variable}
  \item{total_read}{total reading count of miRNA which is produced from different gene locations}
  \item{total_RPM}{total RPM (reading per million) of miRNA}
  }
}
\source{
\url{https://portal.gdc.cancer.gov/}
}
\description{
The dataset contains mirna expression values for tumor tissue sample of TCGA-E9-A1N5 barcoded patient
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_graph.R
\name{vis_graph}
\alias{vis_graph}
\title{Provides visualisation of the graph.}
\usage{
vis_graph(
  input_graph,
  Competing_color = "green",
  mirna_color = "orange",
  Upregulation = "red",
  Downregulation = "blue",
  title = "GRAPH",
  layout = "kk"
)
}
\arguments{
\item{input_graph}{The graph object.}

\item{Competing_color}{The color of competing elements
on the graph with 'green' default.}

\item{mirna_color}{The color of miRNAs on the graph with
'orange' default.}

\item{Upregulation}{The color of Upregulated elements on
the graph with 'red' default.}

\item{Downregulation}{The color of Downregulated elements
on the graph with 'blue' default.}

\item{title}{Title of the given graph.}

\item{layout}{The layout that will be used for
visualisation of the graph.}
}
\value{
The graph object.
}
\description{
`vis_graph` Provides visualisation of the graph.
}
\details{
vis_graph ensures the process to be followed.
}
\examples{

data('minsamp')
data('new_counts')

# Visualisation of graph in steady-state.

 priming_graph(minsamp, Competing_expression, miRNA_expression,
    aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
   vis_graph()

# Visualisation of graph after the change.

 priming_graph(minsamp, Competing_expression, miRNA_expression,
    aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
   update_variables(new_counts)\%>\%
   vis_graph()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_variables.R
\name{update_variables}
\alias{update_variables}
\title{Replaces new values with previous values of
competing or miRNA counts.}
\usage{
update_variables(input_graph, current_counts)
}
\arguments{
\item{input_graph}{The processed graph object.}

\item{current_counts}{The additional df that provided by user.}
}
\value{
the graph object.
}
\description{
This function replaces new values with previous
values of competing or miRNA counts.
}
\details{
update_variables function provides updating edge
variables to current values. If the microRNA or competing
expression (or both) change (decreasing or increasing),
this function switches the values that are found in a new
dataset provided by user. But the current value dataset
must be equal with initial dataset in terms of node name.
}
\examples{

data('minsamp')
data('new_counts')

 minsamp\%>\%
 priming_graph(Competing_expression, miRNA_expression,
     aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
   update_variables(new_counts)
   #new_counts includes the current counts of nodes.

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_node_perturbation.R
\name{find_node_perturbation}
\alias{find_node_perturbation}
\title{Calculates average expression changes of all (or specified)
nodes except trigger and finds the perturbed node count for
all (or specified) nodes in system.}
\usage{
find_node_perturbation(input_graph, how = 2, cycle = 1, limit = 0, fast = 0)
}
\arguments{
\item{input_graph}{The graph object that was processed with priming_graph function.}

\item{how}{The change of count (expression) of the given node in terms of fold change.}

\item{cycle}{The iteration of simulation.}

\item{limit}{The minimum fold change which can be taken into account for
perturbation calculation on all nodes in terms of percentage.}

\item{fast}{specifies percentage of affected target in target expression.
For example, if fast = 1, the nodes that are affected from miRNA repression
activity more than one percent of their expression is determined as subgraph.}
}
\value{
It gives a tibble form dataset that includes node names,
perturbation efficiency and perturbed count of nodes.
}
\description{
Calculates average expression changes of all (or specified)
nodes except trigger and finds the perturbed node count for
all (or specified) nodes in system.
}
\details{
find_node_perturbation calculates mean expression changes
of elements after the change in the network in terms of percentage.
It also calculates the number of nodes that have expression changes
after the change occur in the network. The outputs of the function
are the perturbation efficiency and perturbed count of nodes for
each nodes.
}
\examples{

data('minsamp')
data('midsamp')

 minsamp\%>\%
  priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression)\%>\%
  find_node_perturbation()\%>\%
  select(name, perturbation_efficiency, perturbed_count)


 minsamp\%>\%
  priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression,
    aff_factor = c(energy,seed_type), deg_factor = region)\%>\%
  find_node_perturbation(how = 3, cycle = 4)\%>\%
  select(name, perturbation_efficiency, perturbed_count)

 midsamp\%>\%
  priming_graph(competing_count = Gene_expression, miRNA_count = miRNA_expression)\%>\%
  find_node_perturbation(how = 2, cycle= 3, limit=1, fast = 5)\%>\%
  select(name, perturbation_efficiency, perturbed_count)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priming_graph.R
\name{normalize}
\alias{normalize}
\title{normalize}
\usage{
normalize(x)
}
\arguments{
\item{x}{The variable name that is normalized.}
}
\value{
normalized values
}
\description{
normalizes the values according to maximum values
inside a group. The helper function of priming_graph.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_nodes.R
\name{prepare_rhs_once}
\alias{prepare_rhs_once}
\title{Carries the variables from edge to node.}
\usage{
prepare_rhs_once(input_graph)
}
\arguments{
\item{input_graph}{Processed graph object in previous step.}
}
\value{
tibble object
}
\description{
Carries the variables from edge to node.
}
\details{
The function is a helper function for
processing of graph object with update_nodes function.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_iteration.R
\name{find_iteration}
\alias{find_iteration}
\title{Finds the iteration which provides maximum affected node number}
\usage{
find_iteration(df, limit = 0.1, plot = FALSE)
}
\arguments{
\item{df}{A tbl graph that includes the miRNA and competing targets
triggered and simulated for number of cycles.}

\item{limit}{The minimum amount of change of any node.}

\item{plot}{If TRUE, returns a plot.}
}
\value{
It gives an iteration number to use in simulate() function.
}
\description{
searches the iteration that provides maximum affected node number.
 The user defines a symbolic iteration with .iter. The function
 calculates the number of affected nodes for each iteration and
 then selects the iteration that has maximum affected nodes' number.
}
\examples{

data('midsamp')

midsamp \%>\%
  priming_graph(Gene_expression, miRNA_expression) \%>\%
  update_how('Gene2',2) \%>\%
  simulate(10) \%>\%
  find_iteration(limit=0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{new_counts}
\alias{new_counts}
\title{new_counts}
\format{
A data frame with 7 variables and 7 observation of them:
\describe{
  \item{Competing}{symbol of gene}
  \item{miRNA}{symol of miRNA}
  \item{Competing_count}{Expression values of competing gene}
  \item{miRNA_count}{Expression value of miRNA}
  }
}
\source{
Dataset was created by us.
}
\description{
includes new expression values for minimal sample dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate}
\alias{simulate}
\title{Utilizes the change in expression value/s as triggering.}
\usage{
simulate(input_graph, cycle = 1, threshold = 0, knockdown = TRUE)
}
\arguments{
\item{input_graph}{The graph object that processed in previous steps.}

\item{cycle}{Optimal iteration number for gaining steady-state.}

\item{threshold}{absolute minimum amount of change required to be
considered as up/down regulated element}

\item{knockdown}{specifies gene knockdown with default TRUE}
}
\value{
The graph.
}
\description{
simulate function uses the change in expression value/s as triggering.
}
\details{
The steady-state conditions of the system are disturbed
after the change in the graph (with update_how or update_variables).
In this case, the system tend to be steady state again. The arrangement
of competetive profiles of the targets continue until all nodes are
updated and steady-state nearly. Note that, If `how` argument is
specified as `0`, *simulate()* and *update_how()* functions process
the variables to knockdown of specified gene with default `knockdown = TRUE`
and knocked down competing RNA is kept at zero. However, if `knockdown= FALSE`
argument is applied, competing RNA which has initial expression level of zero
is allowed to increase or fluctuate during calculations.
}
\examples{

data('minsamp')
data('new_counts')

## new_counts, the dataset that includes the current counts of nodes.

priming_graph(minsamp, Competing_expression, miRNA_expression)\%>\%
  update_variables(new_counts)\%>\%
  simulate()

priming_graph(minsamp, Competing_expression, miRNA_expression,
       aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
  update_variables(new_counts)\%>\%
  simulate(cycle = 3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{huge_example}
\alias{huge_example}
\title{huge example}
\format{
A data frame with 7 variables and 26176 observation:
\describe{
  \item{competing}{name of gene}
  \item{miRNA}{name of miRNA}
  \item{competing_counts}{Expression values of competing element (gene)}
  \item{mirnaexpression_normal}{Expression value of miRNA elements in normal tissue}
  \item{Energy}{Energy of miRNA:target binding}
  \item{region_effect}{Coefficient for efficiency of location on target}
  \item{seed_type_effect}{Coefficient for efficiency of seed sequence of miRNA:target interaction}
  }
}
\source{
Dataset was integrated by us.
}
\description{
A sample dataset which is utilised through integration of TCGA_E9_A1N5_normal,
    TCGA_E9_A1N5_mirnanormal and high-throughput experimental miRNA:gene dataset.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{TCGA_E9_A1N5_tumor}
\alias{TCGA_E9_A1N5_tumor}
\title{TCGA_E9_A1N5_tumor}
\format{
Classes tbl_df, tbl and data.frame with 56830 observtion of 7 variables:
\describe{
  \item{patient}{Barcode of patient based on TCGA}
  \item{sample}{Tissue sample barcode of the patient}
  \item{barcode}{Sample barcode of the patient}
  \item{definition}{Tissue type of sample (Primary solid Tumor)}
  \item{ensembl_gene_id}{Gene id}
  \item{external_gene_name}{Gene symbol}
  \item{gene_expression}{Gene expression value}
  }
}
\source{
\url{https://portal.gdc.cancer.gov/}
}
\description{
The dataset contains gene expression values for cancer tissue sample of TCGA-E9-A1N5 barcoded patient
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_perturbation.R
\name{calc_perturbation}
\alias{calc_perturbation}
\title{Calculates average expression changes of all nodes except
trigger and finds the perturbed node count for a given node.}
\usage{
calc_perturbation(input_graph, node_name, how = 1, cycle = 1, limit = 0)
}
\arguments{
\item{input_graph}{the graph object that was processed with
priming graph in previous step.}

\item{node_name}{The node that is trigger for simulation.}

\item{how}{The change of count of the given node in terms of fold change.}

\item{cycle}{The iteration of simulation.}

\item{limit}{The minimum fold change which can be taken into
account for perturbation calculation on all nodes in terms of percentage.}
}
\value{
a tibble with two columns, the perturbation
efficiency and number of perturbed nodes.
}
\description{
Calculates average expression changes of all nodes except
trigger and finds the perturbed node count for a given node.
}
\details{
calc_perturbation calculates mean expression changes
of elements except trigger after the change in the network
in terms of percentage. It also calculates the number of nodes
that have expression changes after the change occur in the network.
The function determines the perturbation efficiency and number
of perturbed nodes after given change with how, cycle and
limit parameter.
}
\examples{

data('minsamp')

minsamp\%>\%
   priming_graph(competing_count = Competing_expression,
       miRNA_count = miRNA_expression)\%>\%
   calc_perturbation('Gene6', how= 3, cycle = 4)

 minsamp\%>\%
   priming_graph(competing_count = Competing_expression, miRNA_count = miRNA_expression,
       aff_factor = c(energy,seed_type), deg_factor = region)\%>\%
   calc_perturbation('Gene6',3, cycle = 4)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{minsamp}
\alias{minsamp}
\title{minsamp}
\format{
A data frame with 7 variables and 7 observation of them:
\describe{
  \item{competing}{symbol of gene}
  \item{miRNA}{symol of miRNA}
  \item{Competing_expression}{Expression values of competing gene}
  \item{miRNA_expression}{Expression value of miRNA}
  \item{seed_type}{Coefficient for efficiency of seed sequence of miRNA:target interaction}
  \item{region}{Coefficient for efficiency of location on target}
  \item{energy}{Energy of miRNA:target binding}
  }
}
\source{
Dataset was created by us.
}
\description{
minimal sample dataset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priming_graph.R
\name{priming_graph}
\alias{priming_graph}
\title{Converts the given dataframe using first variable as
competing and the second as miRNA.
The function converts the given dataframe using first
variable as competing and the second as miRNA. If user
defines interaction factors as affinity or degradation,
the factors are taken into account.}
\usage{
priming_graph(
  df,
  competing_count,
  miRNA_count,
  aff_factor = dummy,
  deg_factor = dummy
)
}
\arguments{
\item{df}{A data frame that includes the miRNA and competing targets.}

\item{competing_count}{The counts (or expression) of competing elements of the dataset.}

\item{miRNA_count}{The counts (or expression) of repressive element (miRNA) of the dataset.}

\item{aff_factor}{The parameter/s of binding between miRNA and targets.}

\item{deg_factor}{The parameter/s for degradation of bound miRNA:target complex.}
}
\value{
the graph object.
}
\description{
Converts the given dataframe using first variable as
competing and the second as miRNA.
The function converts the given dataframe using first
variable as competing and the second as miRNA. If user
defines interaction factors as affinity or degradation,
the factors are taken into account.
}
\details{
priming_graph provides grouping of competing targets
and evaluation of targets within the groups taking into account
miRNA:target, target:total target, interaction and degradation
parameters. The target groups are determined according to miRNAs.
If the factors that are important in target interactions are
specified as arguments, the factors also are evaluated separately
within each group. priming_graph also calculates the miRNA efficiency
in steady-state conditions. It is assumed that quantity of competing
targets and miRNAs are shown in the steady-state system after the
miRNAs exhibit repressive efficiency. Note that the data must not
include missing values such as NA or '-'.
}
\examples{

data('minsamp')

priming_graph(minsamp, Competing_expression, miRNA_expression)

priming_graph(minsamp, Competing_expression, miRNA_expression,
    aff_factor = c(seed_type,energy), deg_factor = region)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_affected_nodes.R
\name{find_affected_nodes}
\alias{find_affected_nodes}
\title{Finds top affected nodes for perturbation 
from a particular node}
\usage{
find_affected_nodes(
  input_graph,
  node_name,
  how = 1,
  cycle = 1,
  limit = 0,
  top = 5
)
}
\arguments{
\item{input_graph}{The graph object that was processed with priming_graph function.}

\item{node_name}{The node to trigger perturbations.}

\item{how}{The change of count (expression) of the given node in terms of fold change.}

\item{cycle}{The iteration of simulation.}

\item{limit}{The minimum fold change which can be taken into account for
perturbation calculation on all nodes in terms of percentage.}

\item{top}{Determines how many nodes most affected will be listed.}
}
\value{
It gives a tibble form dataset that includes perturbation node, 
affected nodes and changes of them.
}
\description{
Finds top affected nodes for perturbation 
from a particular node
}
\details{
Lists the most affected nodes after perturbation 
initiated from a particular node. In the background, 
it compares the calculated values after
the simulation with their initial values.
}
\examples{

data('midsamp')

midsamp\%>\%
priming_graph(competing_count = Gene_expression, 
              miRNA_count = miRNA_expression)\%>\%
find_affected_nodes(node_name = "Gene1", 
                    how = 2, 
                    cycle = 2, 
                    top = 2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_vis.R
\name{simulate_vis}
\alias{simulate_vis}
\title{Provides visualisation of the graph in addition to simulate function.}
\usage{
simulate_vis(
  input_graph,
  cycle = 1,
  threshold = 0,
  save = FALSE,
  Competing_color = "green",
  mirna_color = "orange",
  Upregulation = "red",
  Downregulation = "blue",
  title = "GRAPH",
  layout = "kk"
)
}
\arguments{
\item{input_graph}{The graph object that processed in previous steps.}

\item{cycle}{Optimal iteration number for gaining steady-state.}

\item{threshold}{absolute minimum amount of change required to be considered as up/down regulated element}

\item{save}{provides to save graph output}

\item{Competing_color}{The color of competing elements on the graph with "green" default.}

\item{mirna_color}{The color of miRNAs on the graph with "orange" default.}

\item{Upregulation}{The color of Upregulated elements on the graph with "red" default.}

\item{Downregulation}{The color of Downregulated elements on the graph with "blue" default.}

\item{title}{Title of the given graph.}

\item{layout}{The layout that will be used for visualisation of the graph.}
}
\value{
It gives a graph and the images of states in each iteration until the end of the simulation.
}
\description{
simulate_vis provides visualisation of the graph in addition to simulate function.
}
\details{
simulate_vis gives the last graph object and each iterations' image.
}
\examples{

# When does the system gain steady-state conditions again?

## new_counts, the dataset that includes the current counts of nodes.

data("minsamp")
data("new_counts")

priming_graph(minsamp, Competing_expression, miRNA_expression)\%>\%
  update_variables(new_counts)\%>\%
  simulate_vis()

priming_graph(minsamp, Competing_expression, miRNA_expression, 
    aff_factor = c(seed_type,energy), deg_factor = c(region))\%>\%
  update_variables(new_counts)\%>\%
  simulate_vis(cycle = 12)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_how.R
\name{update_how}
\alias{update_how}
\title{Converts the count value of the given node.}
\usage{
update_how(input_graph, node_name, how, knockdown = TRUE)
}
\arguments{
\item{input_graph}{The graph object that processed in previous step/s.}

\item{node_name}{The name of the node whose count is to be changed.}

\item{how}{The change in terms of fold change.}

\item{knockdown}{specifies gene knockdown with default TRUE}
}
\value{
the graph object.
}
\description{
this function converts the count value of the given node.
}
\details{
update_how function calculates the current value of
given mirna or gene node on the graph object. User must specify
current value as fold change.
}
\examples{

data('minsamp')

priming_graph(minsamp, Competing_expression, miRNA_expression)\%>\%
   update_how('Gene1',3)

 priming_graph(minsamp, Competing_expression, miRNA_expression,
      aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
   update_how('Gene1', 3)

   priming_graph(minsamp, Competing_expression, miRNA_expression,
       aff_factor = c(seed_type,energy), deg_factor = region)\%>\%
  update_how('Gene1', how=0, knockdown= TRUE)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mirtarbasegene}
\alias{mirtarbasegene}
\title{mirtarbasegene}
\format{
Classes tbl_df, tbl and data.frame with 380627 observation of 2 variables:
\describe{
  \item{miRNA}{miRNA symbol}
  \item{Target}{target gene symbol}
  }
}
\source{
\url{http://mirtarbase.mbc.nctu.edu.tw/php/index.php}
}
\description{
the dataset that includes miRNA:target gene interactions downloaded from mirtarbase
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_nodes.R
\name{update_nodes}
\alias{update_nodes}
\title{Carries variables from edge to node.}
\usage{
update_nodes(input_graph, once = FALSE, limit = 0)
}
\arguments{
\item{input_graph}{Processed graph object in previous step.}

\item{once}{The argument is about when the carrying process
runs (internal use only)}

\item{limit}{absolute minimum amount of change required to
be considered as up/down regulated element}
}
\value{
the graph object.
}
\description{
This function carries variables from edge to node and should be used
after `update_how` or `update_variables` functions
}
\details{
If the carrying process performs after priming_graph
function, the argument must be TRUE. The function helps to
visualisation of processed graph object, especially that
includes too many nodes.This step makes it easily to follow
the processes.
}
\examples{

data('minsamp')

minsamp \%>\%
  priming_graph(Competing_expression, miRNA_expression) \%>\%
  update_how('Gene2',2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_targeting_nodes.R
\name{find_targeting_nodes}
\alias{find_targeting_nodes}
\title{Finds potential affecting node for given particular target.}
\usage{
find_targeting_nodes(
  input_graph,
  how = 2,
  cycle = 1,
  limit = 0,
  fast = 0,
  top = 5,
  target = NULL
)
}
\arguments{
\item{input_graph}{The graph object that was processed with priming_graph function.}

\item{how}{The change of count (expression) of the given node in terms of fold change.}

\item{cycle}{The iteration of simulation.}

\item{limit}{The minimum fold change which can be taken into account for
perturbation calculation on all nodes in terms of percentage.}

\item{fast}{specifies percentage of affected target in target expression.
For example, if fast = 1, the nodes that are affected from miRNA repression
activity more than one percent of their expression is determined as subgraph.}

\item{top}{Determines how many nodes most affected will be evaluated.}

\item{target}{The target node in which is being investigated.}
}
\value{
It gives a tibble form dataset that includes parturbation node (source) and
change in count of targeting node
}
\description{
Finds potential affecting node for given particular target.
}
\details{
Lists potential targeting nodes by running 
find_affected_nodes function for all nodes in network.
}
\examples{

data('midsamp')

midsamp\%>\%
priming_graph(competing_count = Gene_expression, 
              miRNA_count = miRNA_expression)\%>\%
find_targeting_nodes(how = 2, 
                     cycle = 2, 
                     target = "Gene1", 
                     top = 2)


}
